@@ -14,9 +14,9 @@ cd `dirname $0`
 
 mkdir -p tools
 cd tools
-cvs -d:pserver:anonymous@dev.eclipse.org:/home/eclipse \
+cvs -d:pserver:anonymous@dev.eclipse.org:/cvsroot/eclipse \
 	checkout org.eclipse.releng.basebuilder
 cd ..
 
-java -jar tools/org.eclipse.releng.basebuilder/startup.jar \
+java -jar tools/org.eclipse.releng.basebuilder/plugins/org.eclipse.equinox.launcher.jar \
 	-ws gtk -application org.eclipse.ant.core.antRunner $*
