@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 QNX Software Systems and others.
+ * Copyright (c) 2000, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -48,7 +48,7 @@ import org.eclipse.cdt.core.model.IParent;
 import org.eclipse.cdt.core.model.ISourceRoot;
 import org.eclipse.cdt.core.model.ITranslationUnit;
 import org.eclipse.cdt.core.model.IWorkingCopy;
-import org.eclipse.cdt.internal.core.search.indexing.IndexManager;
+import org.eclipse.cdt.internal.core.pdom.PDOMManager;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IFolder;
 import org.eclipse.core.resources.IProject;
@@ -63,8 +63,8 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.ISafeRunnable;
 import org.eclipse.core.runtime.Platform;
-import org.eclipse.core.runtime.content.IContentTypeManager.IContentTypeChangeListener;
 import org.eclipse.core.runtime.content.IContentTypeManager.ContentTypeChangeEvent;
+import org.eclipse.core.runtime.content.IContentTypeManager.IContentTypeChangeListener;
 
 public class CModelManager implements IResourceChangeListener, ICDescriptorListener, IContentTypeChangeListener {
 
@@ -199,9 +199,18 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 			}
 			if (res != null && !res.exists()) {
 				res = null;
+			}			
+		}
+
+		// In case this is an external resource see if we can find
+		// a file for it.
+		if (res == null) {
+			IFile[] files = root.findFilesForLocation(path);
+			if (files.length > 0) {
+				res = files[0];
 			}
 		}
-		// TODO: for extenal resources ??
+
 		return create(res, null);
 	}
 
@@ -796,6 +805,11 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 		fire(null, eventType);
 	}
 
+	public void fireShift(ICElement element, int offset, int size, int lines) {
+		ICElementDelta delta = new CShiftData(element, offset, size, lines);
+		fire(delta, ElementChangedEvent.POST_SHIFT); 
+	}
+	
 	/**
 	 * Fire C Model deltas, flushing them after the fact. 
 	 * If the firing mode has been turned off, this has no effect. 
@@ -836,6 +850,9 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 				case ElementChangedEvent.POST_RECONCILE :
 					fireReconcileDelta(listeners, listenerMask, listenerCount);
 					break;
+				case ElementChangedEvent.POST_SHIFT:
+					fireShiftEvent(deltaToNotify, listeners, listenerMask, listenerCount);
+					return;
 			}
 		}
 	}
@@ -879,6 +896,19 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 		}
 	}
 
+	private void fireShiftEvent(ICElementDelta deltaToNotify, IElementChangedListener[] listeners, int[] listenerMask, int listenerCount) {
+
+		// post change deltas
+		if (VERBOSE) {
+			System.out.println("FIRING POST_SHIFT event [" + Thread.currentThread() + "]:"); //$NON-NLS-1$//$NON-NLS-2$
+			System.out.println(deltaToNotify == null ? "<NONE>" : deltaToNotify.toString()); //$NON-NLS-1$
+		}
+		if (deltaToNotify != null) {
+			this.flush();
+			notifyListeners(deltaToNotify, ElementChangedEvent.POST_SHIFT, listeners, listenerMask, listenerCount);
+		}
+	}
+	
 	public void notifyListeners(ICElementDelta deltaToNotify, int eventType,
 		IElementChangedListener[] listeners, int[] listenerMask, int listenerCount) {
 
@@ -1090,10 +1120,6 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 	 *  
 	 */
 	public void shutdown() {
-		if (this.fDeltaProcessor.indexManager != null) { // no more indexing
-			this.fDeltaProcessor.indexManager.shutdown();
-		}
-
 		// Remove ourself from the DescriptorManager.
 		CCorePlugin.getDefault().getCDescriptorManager().removeDescriptorListener(factory);
 		// Remove ourself from the ContentTypeManager
@@ -1108,15 +1134,11 @@ public class CModelManager implements IResourceChangeListener, ICDescriptorListe
 		}
 	}
 
-	public IndexManager getIndexManager() {
-		return this.fDeltaProcessor.indexManager;
-	}
-	
-
 	public void deleting(IProject project) {
-		//	discard all indexing jobs for this project
-		this.getIndexManager().discardJobs(project.getName());
+		// stop the binary runner for this project
 		removeBinaryRunner(project);
+		// stop indexing jobs for this project
+		CCorePlugin.getPDOMManager().deleting(create(project));
 	}
 
-}
\ No newline at end of file
+}
