@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2006 IBM Corporation and others.
+ * Copyright (c) 2004, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -62,6 +62,14 @@ public class CPPASTBinaryExpression extends CPPASTNode implements
         
         if( operand1 != null ) if( !operand1.accept( action ) ) return false;
         if( operand2 != null ) if( !operand2.accept( action ) ) return false;
+        
+        if(action.shouldVisitExpressions ){
+        	switch( action.leave( this ) ){
+        		case ASTVisitor.PROCESS_ABORT : return false;
+        		case ASTVisitor.PROCESS_SKIP  : return true;
+        		default : break;
+        	}
+        }
         return true;
     }
 
