@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2006 IBM Corporation and others.
+ * Copyright (c) 2004, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -62,6 +62,14 @@ public class CPPASTConditionalExpression extends CPPASTNode implements
         if( condition != null ) if( !condition.accept( action ) ) return false;
         if( postive != null ) if( !postive.accept( action ) ) return false;
         if( negative != null ) if( !negative.accept( action ) ) return false;
+        
+        if( action.shouldVisitExpressions ){
+        	switch( action.leave( this ) ){
+        		case ASTVisitor.PROCESS_ABORT : return false;
+        		case ASTVisitor.PROCESS_SKIP  : return true;
+        		default : break;
+        	}
+        }
         return true;
     }
 
