@@ -17,6 +17,7 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 import java.util.StringTokenizer;
+
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.IAddress;
 import org.eclipse.cdt.core.IAddressFactory;
@@ -107,6 +108,7 @@ import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.IExpressionListener;
 import org.eclipse.debug.core.IExpressionManager;
 import org.eclipse.debug.core.ILaunch;
+import org.eclipse.debug.core.ILaunchConfiguration;
 import org.eclipse.debug.core.ILaunchListener;
 import org.eclipse.debug.core.model.IBreakpoint;
 import org.eclipse.debug.core.model.IDebugTarget;
@@ -436,7 +438,7 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	 * 
 	 * @param name the name of this debug target
 	 */
-	protected void setName( String name ) {
+	public void setName( String name ) {
 		fName = name;
 	}
 
@@ -828,7 +830,7 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 			if ( source == null && event instanceof ICDIDestroyedEvent ) {
 				handleTerminatedEvent( (ICDIDestroyedEvent)event );
 			}
-			else if ( source.getTarget().equals( getCDITarget() ) ) {
+			else if ( source != null && source.getTarget().equals( getCDITarget() ) ) {
 				if ( event instanceof ICDICreatedEvent ) {
 					if ( source instanceof ICDIThread ) {
 						handleThreadCreatedEvent( (ICDICreatedEvent)event );
@@ -899,15 +901,18 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 		if ( !canRestart() ) {
 			return;
 		}
-		String mainSymbol = new String( ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
 		try {
-			mainSymbol = getLaunch().getLaunchConfiguration().getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+			ILaunchConfiguration launchConfig = getLaunch().getLaunchConfiguration();
+			if ( launchConfig.getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_DEFAULT ) ) {
+				String mainSymbol = launchConfig.getAttribute( ICDTLaunchConfigurationConstants.ATTR_DEBUGGER_STOP_AT_MAIN_SYMBOL, ICDTLaunchConfigurationConstants.DEBUGGER_STOP_AT_MAIN_SYMBOL_DEFAULT );
+				ICDILocation location = getCDITarget().createFunctionLocation( "", mainSymbol ); //$NON-NLS-1$
+				setInternalTemporaryBreakpoint( location );
+			}
 		}
 		catch( CoreException e ) {
 			requestFailed( e.getMessage(), e );
 		}
-		ICDILocation location = getCDITarget().createFunctionLocation( "", mainSymbol ); //$NON-NLS-1$
-		setInternalTemporaryBreakpoint( location );
+
 		final CDebugElementState newState = CDebugElementState.RESTARTING;
 		changeState( newState );
 		try {
@@ -1134,7 +1139,7 @@ public class CDebugTarget extends CDebugElement implements ICDebugTarget, ICDIEv
 	}
 
 	private void handleSuspendedBySignal( ICDISignalReceived signal ) {
-		fireSuspendEvent( DebugEvent.UNSPECIFIED );
+		fireSuspendEvent( DebugEvent.CLIENT_REQUEST );
 	}
 
 	private void handleErrorInfo( ICDIErrorInfo info ) {
