@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 QNX Software Systems and others.
+ * Copyright (c) 2000, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -59,6 +59,9 @@ import org.eclipse.cdt.debug.mi.core.cdi.SharedLibraryManager;
 import org.eclipse.cdt.debug.mi.core.cdi.SignalManager;
 import org.eclipse.cdt.debug.mi.core.cdi.SourceManager;
 import org.eclipse.cdt.debug.mi.core.cdi.VariableManager;
+import org.eclipse.cdt.debug.mi.core.command.CLIInfoThreads;
+import org.eclipse.cdt.debug.mi.core.command.CLIJump;
+import org.eclipse.cdt.debug.mi.core.command.CLISignal;
 import org.eclipse.cdt.debug.mi.core.command.CommandFactory;
 import org.eclipse.cdt.debug.mi.core.command.MIDataEvaluateExpression;
 import org.eclipse.cdt.debug.mi.core.command.MIExecContinue;
@@ -68,26 +71,63 @@ import org.eclipse.cdt.debug.mi.core.command.MIExecRun;
 import org.eclipse.cdt.debug.mi.core.command.MIExecStep;
 import org.eclipse.cdt.debug.mi.core.command.MIExecStepInstruction;
 import org.eclipse.cdt.debug.mi.core.command.MIExecUntil;
-import org.eclipse.cdt.debug.mi.core.command.CLIInfoThreads;
-import org.eclipse.cdt.debug.mi.core.command.CLIJump;
 import org.eclipse.cdt.debug.mi.core.command.MIGDBShowEndian;
-import org.eclipse.cdt.debug.mi.core.command.CLISignal;
 import org.eclipse.cdt.debug.mi.core.command.MITargetDetach;
 import org.eclipse.cdt.debug.mi.core.command.MIThreadSelect;
 import org.eclipse.cdt.debug.mi.core.event.MIDetachedEvent;
 import org.eclipse.cdt.debug.mi.core.event.MIThreadCreatedEvent;
 import org.eclipse.cdt.debug.mi.core.event.MIThreadExitEvent;
+import org.eclipse.cdt.debug.mi.core.output.CLIInfoThreadsInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIDataEvaluateExpressionInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIFrame;
-import org.eclipse.cdt.debug.mi.core.output.MIInfo;
-import org.eclipse.cdt.debug.mi.core.output.CLIInfoThreadsInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIGDBShowEndianInfo;
+import org.eclipse.cdt.debug.mi.core.output.MIInfo;
 import org.eclipse.cdt.debug.mi.core.output.MIThreadSelectInfo;
+import org.eclipse.core.runtime.Path;
 
 /**
  */
 public class Target extends SessionObject implements ICDITarget {
 
+	public class Lock {
+
+		java.lang.Thread heldBy;
+		int count;
+
+		public Lock() {
+
+		}
+
+		public synchronized void aquire() {
+			if (heldBy == null || heldBy == java.lang.Thread.currentThread()) {
+				heldBy = java.lang.Thread.currentThread();
+				count++;
+			} else {
+				while (true) {
+					try {
+						wait();
+					} catch (InterruptedException e) {
+					}
+					if (heldBy == null) {
+						heldBy = java.lang.Thread.currentThread();
+						count++;
+						return;
+					}					
+				}
+			}
+		}
+
+		public synchronized void release() {
+			if (heldBy == null || heldBy != java.lang.Thread.currentThread()) {
+				throw new IllegalStateException("Thread does not own lock");
+			}
+			if(--count == 0) {
+				heldBy = null;
+				notifyAll();
+			}
+		}
+	}
+
 	MISession miSession;
 	ICDITargetConfiguration fConfiguration;
 	Thread[] noThreads = new Thread[0];
@@ -95,13 +135,23 @@ public class Target extends SessionObject implements ICDITarget {
 	int currentThreadId;
 	String fEndian = null;
 	boolean suspended = true;
-	
+	boolean deferBreakpoints = true;
+	Lock lock = new Lock();
+
 	public Target(Session s, MISession mi) {
 		super(s);
 		miSession = mi;
 		currentThreads = noThreads;
 	}
 
+	public void lockTarget() {
+		lock.aquire();
+	}
+	
+	public void releaseTarget() {
+		lock.release();
+	}
+	
 	public MISession getMISession() {
 		return miSession;
 	}
@@ -109,13 +159,14 @@ public class Target extends SessionObject implements ICDITarget {
 	public void setConfiguration(ICDITargetConfiguration configuration) {
 		fConfiguration = configuration;
 	}
+
 	/**
 	 * @see org.eclipse.cdt.debug.core.cdi.model.ICDIObject#getTarget()
 	 */
 	public ICDITarget getTarget() {
 		return this;
 	}
-	
+
 	/**
 	 * @see org.eclipse.cdt.debug.core.cdi.model.ICDITarget#setCurrentThread(ICDIThread)
 	 */
@@ -126,7 +177,7 @@ public class Target extends SessionObject implements ICDITarget {
 			throw new CDIException(CdiResources.getString("cdi.model.Target.Unknown_thread")); //$NON-NLS-1$
 		}
 	}
-	
+
 	public void setCurrentThread(ICDIThread cthread, boolean doUpdate) throws CDIException {
 		if (cthread instanceof Thread) {
 			setCurrentThread((Thread)cthread, doUpdate);
@@ -162,7 +213,7 @@ public class Target extends SessionObject implements ICDITarget {
 				currentThreadId = info.getNewThreadId();
 
 				// @^&#@^$*^$
-				// GDB reset the currentFrame to  some other level 0 when switching thread.
+				// GDB reset the currentFrame to some other level 0 when switching thread.
 				// we need to reposition the current stack level.
 				MIFrame miFrame = info.getFrame();
 				if (miFrame != null) {
@@ -175,7 +226,7 @@ public class Target extends SessionObject implements ICDITarget {
 
 			Session session = (Session)getSession();
 			// Resetting threads may change the value of
-			// some variables like Register.  Call an update()
+			// some variables like Register. Call an update()
 			// To generate changeEvents.
 			if (doUpdate) {
 				RegisterManager regMgr = session.getRegisterManager();
@@ -191,7 +242,7 @@ public class Target extends SessionObject implements ICDITarget {
 
 		// We should be allright now.
 		if (currentThreadId != id) {
-			// thread is gone.  Generate a Thread destroyed.
+			// thread is gone. Generate a Thread destroyed.
 			miSession.fireEvent(new MIThreadExitEvent(miSession, id));
 			throw new CDIException(CdiResources.getString("cdi.model.Target.Cannot_switch_to_thread") + id); //$NON-NLS-1$
 		}
@@ -204,8 +255,9 @@ public class Target extends SessionObject implements ICDITarget {
 		Thread[] oldThreads = currentThreads;
 
 		// If we use "info threads" in getCThreads() this
-		// will be overwritten.  However if we use -stack-list-threads
+		// will be overwritten. However if we use -stack-list-threads
 		// it does not provide to the current thread
+		lockTarget();
 		currentThreadId = newThreadId;
 
 		// get the new Threads.
@@ -214,6 +266,7 @@ public class Target extends SessionObject implements ICDITarget {
 		} catch (CDIException e) {
 			currentThreads = noThreads;
 		}
+		releaseTarget();
 
 		// Fire CreatedEvent for new threads.
 		// Replace the new threads with the old thread object
@@ -272,6 +325,7 @@ public class Target extends SessionObject implements ICDITarget {
 	public Thread[] getCThreads() throws CDIException {
 		Thread[] cthreads = noThreads;
 		try {
+			lockTarget();
 			RxThread rxThread = miSession.getRxThread();
 			rxThread.setEnableConsole(false);
 			CommandFactory factory = miSession.getCommandFactory();
@@ -279,11 +333,11 @@ public class Target extends SessionObject implements ICDITarget {
 			// HACK/FIXME: gdb/mi thread-list-ids does not
 			// show any newly create thread, we workaround by
 			// issuing "info threads" instead.
-			//MIThreadListIds tids = factory.createMIThreadListIds();
-			//MIThreadListIdsInfo info = tids.getMIThreadListIdsInfo();
+			// MIThreadListIds tids = factory.createMIThreadListIds();
+			// MIThreadListIdsInfo info = tids.getMIThreadListIdsInfo();
 			miSession.postCommand(tids);
 			CLIInfoThreadsInfo info = tids.getMIInfoThreadsInfo();
-			int [] ids;
+			int[] ids;
 			String[] names;
 			if (info == null) {
 				ids = new int[0];
@@ -291,6 +345,7 @@ public class Target extends SessionObject implements ICDITarget {
 			} else {
 				ids = info.getThreadIds();
 				names = info.getThreadNames();
+				currentThreadId = info.getCurrentThread();
 			}
 			if (ids != null && ids.length > 0) {
 				cthreads = new Thread[ids.length];
@@ -308,11 +363,10 @@ public class Target extends SessionObject implements ICDITarget {
 				// Provide a dummy.
 				cthreads = new Thread[]{new Thread(this, 0)};
 			}
-			currentThreadId = info.getCurrentThread();
-			//FIX: When attaching there is no thread selected
+			// FIX: When attaching there is no thread selected
 			// We will choose the first one as a workaround.
 			if (currentThreadId == 0 && cthreads.length > 0) {
-				currentThreadId = cthreads[0].getId();
+				setCurrentThread(cthreads[0], false);				
 			}
 		} catch (MIException e) {
 			// Do not throw anything in this case.
@@ -320,6 +374,7 @@ public class Target extends SessionObject implements ICDITarget {
 		} finally {
 			RxThread rxThread = miSession.getRxThread();
 			rxThread.setEnableConsole(true);
+			releaseTarget();
 		}
 		return cthreads;
 	}
@@ -415,10 +470,9 @@ public class Target extends SessionObject implements ICDITarget {
 			}
 		} catch (MIException e) {
 			throw new MI2CDIException(e);
-		}		
+		}
 	}
 
-
 	/**
 	 * @see org.eclipse.cdt.debug.core.cdi.model.ICDITarget#stepIntoInstruction()
 	 */
@@ -441,7 +495,7 @@ public class Target extends SessionObject implements ICDITarget {
 		} catch (MIException e) {
 			throw new MI2CDIException(e);
 		}
-		
+
 	}
 
 	/**
@@ -465,7 +519,7 @@ public class Target extends SessionObject implements ICDITarget {
 			}
 		} catch (MIException e) {
 			throw new MI2CDIException(e);
-		}		
+		}
 	}
 
 	/**
@@ -528,14 +582,14 @@ public class Target extends SessionObject implements ICDITarget {
 				}
 			}
 		} else if (location instanceof ICDIAddressLocation) {
-			ICDIAddressLocation addrLocation = (ICDIAddressLocation) location;
-			if (! addrLocation.getAddress().equals(BigInteger.ZERO)) {
+			ICDIAddressLocation addrLocation = (ICDIAddressLocation)location;
+			if (!addrLocation.getAddress().equals(BigInteger.ZERO)) {
 				loc = "*0x" + addrLocation.getAddress().toString(16); //$NON-NLS-1$
 			}
 		}
 		// Throw an exception we do know where to go
 		if (loc == null) {
-			throw new CDIException (CdiResources.getString("cdi.mode.Target.Bad_location")); //$NON-NLS-1$
+			throw new CDIException(CdiResources.getString("cdi.mode.Target.Bad_location")); //$NON-NLS-1$
 		}
 		MIExecUntil until = factory.createMIExecUntil(loc);
 		try {
@@ -546,7 +600,7 @@ public class Target extends SessionObject implements ICDITarget {
 			}
 		} catch (MIException e) {
 			throw new MI2CDIException(e);
-		}		
+		}
 	}
 
 	/**
@@ -579,7 +633,7 @@ public class Target extends SessionObject implements ICDITarget {
 				((EventManager)getSession().getEventManager()).allowProcessingEvents(false);
 				suspend();
 			} finally {
-				((EventManager)getSession().getEventManager()).allowProcessingEvents(true);			
+				((EventManager)getSession().getEventManager()).allowProcessingEvents(true);
 			}
 		}
 		CommandFactory factory = miSession.getCommandFactory();
@@ -650,7 +704,7 @@ public class Target extends SessionObject implements ICDITarget {
 			}
 		} catch (MIException e) {
 			throw new MI2CDIException(e);
-		}		
+		}
 	}
 
 	/**
@@ -675,14 +729,14 @@ public class Target extends SessionObject implements ICDITarget {
 				}
 			}
 		} else if (location instanceof ICDIAddressLocation) {
-			ICDIAddressLocation addrLocation = (ICDIAddressLocation) location;
-			if (! addrLocation.getAddress().equals(BigInteger.ZERO)) {
+			ICDIAddressLocation addrLocation = (ICDIAddressLocation)location;
+			if (!addrLocation.getAddress().equals(BigInteger.ZERO)) {
 				loc = "*0x" + addrLocation.getAddress().toString(16); //$NON-NLS-1$
 			}
 		}
 		// Throw an exception we do know where to go
 		if (loc == null) {
-			throw new CDIException (CdiResources.getString("cdi.mode.Target.Bad_location")); //$NON-NLS-1$
+			throw new CDIException(CdiResources.getString("cdi.mode.Target.Bad_location")); //$NON-NLS-1$
 		}
 
 		CLIJump jump = factory.createCLIJump(loc);
@@ -811,6 +865,10 @@ public class Target extends SessionObject implements ICDITarget {
 	public ICDILineBreakpoint setLineBreakpoint(int type, ICDILineLocation location,
 			ICDICondition condition, boolean deferred) throws CDIException {		
 		BreakpointManager bMgr = ((Session)getSession()).getBreakpointManager();
+		// See bug 102563. Only pass gdb the source file name, not the full path.
+		// This workaround was in CBreakpointManager in CDT 3.1 but has been moved
+		// here so that non-gdb CDI clients can get the full path.
+		location = createLineLocation( new Path(location.getFile()).lastSegment(), location.getLineNumber() );
 		return bMgr.setLineBreakpoint(this, type, location, condition, deferred);
 	}
 
@@ -870,10 +928,10 @@ public class Target extends SessionObject implements ICDITarget {
 	 */
 	public void deleteAllBreakpoints() throws CDIException {
 		BreakpointManager bMgr = ((Session)getSession()).getBreakpointManager();
-		bMgr.deleteAllBreakpoints(this);		
+		bMgr.deleteAllBreakpoints(this);
 	}
 
-	/* 
+	/*
 	 * @see org.eclipse.cdt.debug.core.cdi.model.ICDITarget#createCondition(int, java.lang.String, String)
 	 */
 	public ICDICondition createCondition(int ignoreCount, String expression) {
@@ -1089,14 +1147,14 @@ public class Target extends SessionObject implements ICDITarget {
 		if (fConfiguration == null) {
 			if (miSession.isProgramSession()) {
 				fConfiguration = new TargetConfiguration(this);
-			}  else if (miSession.isAttachSession()){
+			} else if (miSession.isAttachSession()) {
 				fConfiguration = new TargetConfiguration(this);
 			} else if (miSession.isCoreSession()) {
 				fConfiguration = new CoreFileConfiguration(this);
 			} else {
-				fConfiguration = new TargetConfiguration(this);				
+				fConfiguration = new TargetConfiguration(this);
 			}
-		}		
+		}
 		return fConfiguration;
 	}
 
@@ -1118,10 +1176,24 @@ public class Target extends SessionObject implements ICDITarget {
 		if (varDesc instanceof RegisterDescriptor) {
 			Session session = (Session)getTarget().getSession();
 			RegisterManager mgr = session.getRegisterManager();
-			return mgr.createRegister((RegisterDescriptor)varDesc);			
+			return mgr.createRegister((RegisterDescriptor)varDesc);
 		}
-		return null;		
+		return null;
 	}
 
+	public void deferBreakpoints(boolean defer) {
+		this.deferBreakpoints = defer;
+	}
 
+	public boolean areBreakpointsDeferred() {
+		return this.deferBreakpoints;
+	}
+
+	public void enableVerboseMode(boolean enabled) {
+		miSession.enableVerboseMode(enabled);
+	}
+
+	public boolean isVerboseModeEnabled() {
+		return miSession.isVerboseModeEnabled();
+	}
 }
