@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -16,6 +16,7 @@ import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IPath;
 
 import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGenerator;
+import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGeneratorType;
 
 /**
  * This interface represents a utility of some sort that is used in the build process.
@@ -45,7 +46,8 @@ public interface ITool extends IBuildObject, IHoldsOptions {
 	
 	public static final String VERSIONS_SUPPORTED = "versionsSupported";	//$NON-NLS-1$
 	public static final String CONVERT_TO_ID = "convertToId";				//$NON-NLS-1$
-	
+	public static final String OPTIONPATHCONVERTER = "optionPathConverter";				//$NON-NLS-1$
+
 	public static final int FILTER_C = 0;
 	public static final int FILTER_CC = 1;
 	public static final int FILTER_BOTH = 2;
@@ -547,9 +549,9 @@ public interface ITool extends IBuildObject, IHoldsOptions {
 	 * when the project is built.
 	 *
 	 * @param sourceExt  source file extension
-	 * @return IManagedDependencyGenerator
+	 * @return IManagedDependencyGeneratorType
 	 */
-	public IManagedDependencyGenerator getDependencyGeneratorForExtension(String sourceExt);
+	public IManagedDependencyGeneratorType getDependencyGeneratorForExtension(String sourceExt);
 	
 	/**
 	 * Returns an array of command line arguments that have been specified for
@@ -722,4 +724,10 @@ public interface ITool extends IBuildObject, IHoldsOptions {
 	 * @return IEnvVarBuildPath[]
 	 */
 	public IEnvVarBuildPath[] getEnvVarBuildPaths();
+	
+	/**
+	 * Returns an IOptionPathConverter implementation for this tool
+	 * or null, if no conversion is required
+	 */
+	public IOptionPathConverter getOptionPathConverter() ;
 }
