@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -35,6 +35,7 @@ import org.eclipse.cdt.managedbuilder.core.IResourceConfiguration;
 import org.eclipse.cdt.managedbuilder.core.ITool;
 import org.eclipse.cdt.managedbuilder.core.IToolChain;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
+import org.eclipse.cdt.managedbuilder.core.ManagedBuilderCorePlugin;
 import org.eclipse.cdt.managedbuilder.envvar.IConfigurationEnvironmentVariableSupplier;
 import org.eclipse.cdt.managedbuilder.internal.envvar.EnvironmentVariableProvider;
 import org.eclipse.cdt.managedbuilder.internal.envvar.UserDefinedEnvironmentSupplier;
@@ -44,9 +45,13 @@ import org.eclipse.cdt.managedbuilder.macros.IConfigurationBuildMacroSupplier;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceDelta;
+import org.eclipse.core.resources.ProjectScope;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.PluginVersionIdentifier;
+import org.osgi.service.prefs.BackingStoreException;
+import org.osgi.service.prefs.Preferences;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -80,6 +85,44 @@ public class Configuration extends BuildObject implements IConfiguration {
 	private boolean resolved = true;
 	private boolean isTemporary = false;
 
+	//property name for holding the rebuild state
+	private static final String REBUILD_STATE = "rebuildState";  //$NON-NLS-1$
+
+	//The resource delta passed to the builder is not always up-to-date
+	//for the given configuration because between two builds of the same configuration
+	//any number of other configuration builds may occur
+	//that is why we need to keep some information regarding what happened
+	//with the resource tree between the two configuration builds
+	//
+	//The trivial approach implemented currently is to hold
+	//the general information of whether some resources were 
+	//removed,changed,etc. and detect whether the rebuild is needed
+	//based upon this information
+	//
+	//In the future we might implement some more smart mechanism
+	//for tracking delta, e.g calculate the pre-cinfiguration resource delta, etc.
+	//
+	//property for holding the resource change state
+	private static final String RC_CHANGE_STATE = "rcState";  //$NON-NLS-1$
+	//resource change state
+	private int resourceChangeState;
+
+	//Internal Builder state
+	//NOTE: these are temporary properties
+	//In the future we are going present the Internal Builder
+	//as a special Builder object of the tool-chain and implement the internal
+	//builder enabling/disabling as the Builder substitution functionality
+	//
+	private static final String INTERNAL_BUILDER = "internalBuilder";
+	//preference key that holds the Internal Builder enable state 
+	private static final String INTERNAL_BUILDER_ENABLED = "enabled";  //$NON-NLS-1$
+	//preference key that holds the internal builder mode
+	private static final String INTERNAL_BUILDER_IGNORE_ERR = "ignoreErr";  //$NON-NLS-1$
+	//Internal Builder enable state
+	private boolean internalBuilderEnabled;
+	//Internal Builder mode
+	private boolean internalBuilderIgnoreErr = true;
+	
 	
 	/*
 	 *  C O N S T R U C T O R S
@@ -215,6 +258,29 @@ public class Configuration extends BuildObject implements IConfiguration {
 				addResourceConfiguration(resConfig);
 			}
 		}
+		
+		PropertyManager mngr = PropertyManager.getInstance();
+		String rebuild = mngr.getProperty(this, REBUILD_STATE);
+		if(rebuild == null || Boolean.valueOf(rebuild).booleanValue())
+			rebuildNeeded = true;
+		
+		String rcChangeState = mngr.getProperty(this, RC_CHANGE_STATE);
+		if(rcChangeState == null)
+			resourceChangeState = ~0;
+		else {
+			try {
+			resourceChangeState = Integer.parseInt(rcChangeState);
+			} catch (NumberFormatException e){
+				resourceChangeState = ~0;
+			}
+		}
+		
+//		Preferences prefs = getPreferences(INTERNAL_BUILDER);
+//		
+//		internalBuilderEnabled = prefs != null ?
+//				prefs.getBoolean(INTERNAL_BUILDER_ENABLED, false) : false;
+//		internalBuilderIgnoreErr = prefs != null ?
+//				prefs.getBoolean(INTERNAL_BUILDER_IGNORE_ERR, true) : true;
 	}
 
 	/**
@@ -267,6 +333,11 @@ public class Configuration extends BuildObject implements IConfiguration {
 			postannouncebuildStep = new String(cloneConfig.postannouncebuildStep);
 		} 
 		
+		enableInternalBuilder(cloneConfig.isInternalBuilderEnabled());
+		setInternalBuilderIgnoreErr(cloneConfig.getInternalBuilderIgnoreErr());
+//		internalBuilderEnabled = cloneConfig.internalBuilderEnabled;
+//		internalBuilderIgnoreErr = cloneConfig.internalBuilderIgnoreErr;
+		
 		// Clone the configuration's children
 		// Tool Chain
 		String subId;
@@ -337,7 +408,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 			Iterator iter = resElements.listIterator();
 			while (iter.hasNext()) {
 				ResourceConfiguration resConfig = (ResourceConfiguration) iter.next();
-				subId = getId() + "." + ManagedBuildManager.getRandomNumber(); //$NON-NLS-1$
+				subId = ManagedBuildManager.calculateChildId(getId(), resConfig.getResourcePath());
 				ResourceConfiguration newResConfig = new ResourceConfiguration(this, resConfig, subId);
 				addResourceConfiguration(newResConfig);
 			}
@@ -346,7 +417,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 		// Hook me up
 		managedProject.addConfiguration(this);
 		setDirty(true);
-		rebuildNeeded = true;
+		setRebuildState(true);
 	}
 
 	/*
@@ -522,6 +593,8 @@ public class Configuration extends BuildObject implements IConfiguration {
 			element.appendChild(resElement);
 			resConfig.serialize(doc, resElement);
 		}
+
+		PropertyManager.getInstance().serialize(this);
 		
 		// I am clean now
 		isDirty = false;
@@ -660,6 +733,13 @@ public class Configuration extends BuildObject implements IConfiguration {
 		return toolChain.getTool(id);
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IConfiguration#getToolsBySuperClassId(java.lang.String) 
+	 */
+	public ITool[] getToolsBySuperClassId(String id) {
+		return toolChain.getToolsBySuperClassId(id);
+	}
+
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.managedbuilder.core.IConfiguration#getTargetTool()
 	 */
@@ -710,28 +790,9 @@ public class Configuration extends BuildObject implements IConfiguration {
 		// Is there a change?
 		IOption retOpt = option;
 		if (option.getBooleanValue() != value) {
-			if (option.isExtensionElement()) {
-				//  If the extension element was created from an MBS 2.0 model OptionReference element, hook the
-				//  new option up to its superclass directly.  This is to avoid references to oddly id'ed
-				//  elements that are automatically generated from V2.0 model optionReferences.  If these
-				//  end up in the project file, then the project could have a problem when the integration
-				//  provider switches to providing the new model.
-				IOption newSuperClass = option;
-				if (((Option)option).wasOptRef()) {
-					newSuperClass = option.getSuperClass();
-				}
-				//  Create an Option element for the managed build project file (.CDTBUILD)
-				String subId;
-				int nnn = ManagedBuildManager.getRandomNumber();
-				subId = newSuperClass.getId() + "." + nnn; //$NON-NLS-1$
-				retOpt = holder.createOption(newSuperClass, subId, null, false); 
-				retOpt.setValueType(option.getValueType());
-				retOpt.setValue(value);
-				setDirty(true);
-			} else {
-				option.setValue(value);
-			}
-			rebuildNeeded = true;
+			retOpt = holder.getOptionToSet(option, false);
+			retOpt.setValue(value);
+//			rebuildNeeded = true;
 		}
 		return retOpt;
 	}
@@ -744,28 +805,9 @@ public class Configuration extends BuildObject implements IConfiguration {
 		String oldValue;
 		oldValue = option.getStringValue(); 
 		if (oldValue != null && !oldValue.equals(value)) {
-			if (option.isExtensionElement()) {
-				//  If the extension element was created from an MBS 2.0 model OptionReference element, hook the
-				//  new option up to its superclass directly.  This is to avoid references to oddly id'ed
-				//  elements that are automatically generated from V2.0 model optionReferences.  If these
-				//  end up in the project file, then the project could have a problem when the integration
-				//  provider switches to providing the new model.
-				IOption newSuperClass = option;
-				if (((Option)option).wasOptRef()) {
-					newSuperClass = option.getSuperClass();
-				}
-				//  Create an Option element for the managed build project file (.CDTBUILD)
-				String subId;
-				int nnn = ManagedBuildManager.getRandomNumber();
-				subId = newSuperClass.getId() + "." + nnn; //$NON-NLS-1$
-				retOpt = holder.createOption(newSuperClass, subId, null, false); 
-				retOpt.setValueType(option.getValueType());
-				retOpt.setValue(value);
-				setDirty(true);
-			} else {
-				option.setValue(value);
-			}
-			rebuildNeeded = true;
+			retOpt = holder.getOptionToSet(option, false);
+			retOpt.setValue(value);
+//			rebuildNeeded = true;
 		}
 		return retOpt;
 	}
@@ -798,28 +840,9 @@ public class Configuration extends BuildObject implements IConfiguration {
 				break;
 		}
 		if(!Arrays.equals(value, oldValue)) {
-			if (option.isExtensionElement()) {
-				//  If the extension element was created from an MBS 2.0 model OptionReference element, hook the
-				//  new option up to its superclass directly.  This is to avoid references to oddly id'ed
-				//  elements that are automatically generated from V2.0 model optionReferences.  If these
-				//  end up in the project file, then the project could have a problem when the integration
-				//  provider switches to providing the new model.
-				IOption newSuperClass = option;
-				if (((Option)option).wasOptRef()) {
-					newSuperClass = option.getSuperClass();
-				}
-				//  Create an Option element for the managed build project file (.CDTBUILD)
-				String subId;
-				int nnn = ManagedBuildManager.getRandomNumber();
-				subId = newSuperClass.getId() + "." + nnn; //$NON-NLS-1$
-				retOpt = holder.createOption(newSuperClass, subId, null, false); 
-				retOpt.setValueType(option.getValueType());
-				retOpt.setValue(value);
-				setDirty(true);
-			} else {
-				option.setValue(value);
-			}
-			rebuildNeeded = true;
+			retOpt = holder.getOptionToSet(option, false);
+			retOpt.setValue(value);
+//			rebuildNeeded = true;
 		} 
 		return retOpt;
 	}
@@ -857,10 +880,12 @@ public class Configuration extends BuildObject implements IConfiguration {
 		getResourceConfigurationList().add(resConfig);
 		getResourceConfigurationMap().put(resConfig.getResourcePath(), resConfig);
 		isDirty = true;
-		rebuildNeeded = true;
+//		rebuildNeeded = true;
 	}
 
 	public void removeResourceConfiguration(IResourceConfiguration resConfig) {
+		ManagedBuildManager.performValueHandlerEvent(resConfig, 
+ 					IManagedOptionValueHandler.EVENT_CLOSE);
 		getResourceConfigurationList().remove(resConfig);
 		getResourceConfigurationMap().remove(resConfig.getResourcePath());
 		isDirty = true;
@@ -1022,7 +1047,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 				return parent.getCleanCommand();
 			} else {
 				// User forgot to specify it. Guess based on OS.
-				if (Platform.getOS().equals("OS_WIN32")) { //$NON-NLS-1$
+				if (Platform.getOS().equals(Platform.OS_WIN32)) {
 					return new String("del"); //$NON-NLS-1$
 				} else {
 					return new String("rm"); //$NON-NLS-1$
@@ -1108,7 +1133,15 @@ public class Configuration extends BuildObject implements IConfiguration {
 		if (extension == null && artifactExtension == null) return;
 		if (artifactExtension == null || extension == null || !artifactExtension.equals(extension)) {
 			artifactExtension = extension;
-			rebuildNeeded = true;
+//			rebuildNeeded = true;
+			if(!isExtensionElement()){
+				ITool tool = calculateTargetTool();
+				if(tool != null){
+					tool.setRebuildState(true);
+				} else {
+					setRebuildState(true);
+				}
+			}
 			isDirty = true;
 		}
 	}
@@ -1120,7 +1153,15 @@ public class Configuration extends BuildObject implements IConfiguration {
 		if (name == null && artifactName == null) return;
 		if (artifactName == null || name == null || !artifactName.equals(name)) {
 			artifactName = name;
-			rebuildNeeded = true;
+			if(!isExtensionElement()){
+				ITool tool = calculateTargetTool();
+				if(tool != null) {
+					tool.setRebuildState(true);
+				} else {
+					setRebuildState(true);
+				}
+			}
+//			rebuildNeeded = true;
 			isDirty = true;
 		}
 	}
@@ -1168,7 +1209,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 		if(makeArgs == null){ //resetting the build arguments
 			if(!builder.isExtensionElement()){
 				builder.setArguments(makeArgs);
-				rebuildNeeded = true;
+//				rebuildNeeded = true;
 			}
 		}else if(!makeArgs.equals(builder.getArguments())){
 			if (builder.isExtensionElement()) {
@@ -1177,7 +1218,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 				builder = toolChain.createBuilder(builder, subId, builderName, false);
 			}
 			builder.setArguments(makeArgs);
-			rebuildNeeded = true;
+//			rebuildNeeded = true;
 		}
 	}
 
@@ -1190,7 +1231,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 		if(command == null){ //resetting the build command
 			if(!builder.isExtensionElement()){
 				builder.setCommand(command);
-				rebuildNeeded = true;
+//				rebuildNeeded = true;
 			}
 		} else if(!command.equals(builder.getCommand())){
 			if (builder.isExtensionElement()) {
@@ -1199,7 +1240,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 				builder = toolChain.createBuilder(builder, subId, builderName, false);
 			}
 			builder.setCommand(command);
-			rebuildNeeded = true;
+//			rebuildNeeded = true;
 		}
 	}
  
@@ -1210,7 +1251,7 @@ public class Configuration extends BuildObject implements IConfiguration {
         if (step == null && prebuildStep == null) return; 
         if (prebuildStep == null || step == null || !prebuildStep.equals(step)) { 
             prebuildStep = step; 
-            rebuildNeeded = true;
+//			rebuildNeeded = true;
             isDirty = true; 
         } 
     } 
@@ -1223,7 +1264,7 @@ public class Configuration extends BuildObject implements IConfiguration {
         if (step == null && postbuildStep == null) return; 
         if (postbuildStep == null || step == null || !postbuildStep.equals(step)) { 
             postbuildStep = step; 
-    		rebuildNeeded = true;
+//    		rebuildNeeded = true;
             isDirty = true; 
         }       
     } 
@@ -1235,7 +1276,7 @@ public class Configuration extends BuildObject implements IConfiguration {
         if (announceStep == null && preannouncebuildStep == null) return; 
         if (preannouncebuildStep == null || announceStep == null || !preannouncebuildStep.equals(announceStep)) {
             preannouncebuildStep = announceStep; 
-    		rebuildNeeded = true;
+//    		rebuildNeeded = true;
             isDirty = true; 
         } 
     } 
@@ -1247,7 +1288,7 @@ public class Configuration extends BuildObject implements IConfiguration {
         if (announceStep == null && postannouncebuildStep == null) return; 
         if (postannouncebuildStep == null || announceStep == null || !postannouncebuildStep.equals(announceStep)) {
             postannouncebuildStep = announceStep; 
-    		rebuildNeeded = true;
+//    		rebuildNeeded = true;
             isDirty = true; 
         } 
     } 
@@ -1261,6 +1302,46 @@ public class Configuration extends BuildObject implements IConfiguration {
 			return toolChain.isSupported();
 		return false;
 	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IConfiguration#isHeaderFile(java.lang.String)
+	 */
+	public boolean isHeaderFile(String ext) {
+		// Check to see if there is a rule to build a file with this extension
+		IManagedProject manProj = getManagedProject();
+		IProject project = null;
+		if (manProj != null) {
+			project = (IProject)manProj.getOwner();
+		}
+		ITool[] tools = getFilteredTools();
+		for (int index = 0; index < tools.length; index++) {
+			ITool tool = tools[index];
+			try {
+				if (project != null) {
+					// Make sure the tool is right for the project
+					switch (tool.getNatureFilter()) {
+						case ITool.FILTER_C:
+							if (project.hasNature(CProjectNature.C_NATURE_ID) && !project.hasNature(CCProjectNature.CC_NATURE_ID)) {
+								return tool.isHeaderFile(ext);
+							}
+							break;
+						case ITool.FILTER_CC:
+							if (project.hasNature(CCProjectNature.CC_NATURE_ID)) {
+								return tool.isHeaderFile(ext);
+							}
+							break;
+						case ITool.FILTER_BOTH:
+							return tool.isHeaderFile(ext);
+					}
+				} else {
+					return tool.isHeaderFile(ext);
+				}
+			} catch (CoreException e) {
+				continue;
+			}
+		}
+		return false;
+	}
 
 	/*
 	 *  O B J E C T   S T A T E   M A I N T E N A N C E
@@ -1298,7 +1379,41 @@ public class Configuration extends BuildObject implements IConfiguration {
 	 * @see org.eclipse.cdt.managedbuilder.core.IConfiguration#needsRebuild()
 	 */
 	public boolean needsRebuild() {
-		return rebuildNeeded;
+		return needsRebuild(true);
+	}
+	
+	public boolean needsFullRebuild() {
+		return needsRebuild(false);
+	}
+	
+	public boolean needsRebuild(boolean checkChildren) {
+		boolean needRebuild = rebuildNeeded || resourceChangesRequireRebuild(); 
+		
+		if(needRebuild || !checkChildren)
+			return needRebuild;
+		
+		if(toolChain.needsRebuild())
+			return true;
+		
+		for(Iterator iter = resourceConfigurationList.iterator();iter.hasNext();){
+			IResourceConfiguration rcCfg = (IResourceConfiguration)iter.next();
+			if(rcCfg.needsRebuild())
+				return true;
+			
+			ITool tools[] = rcCfg.getToolsToInvoke();
+			for(int i = 0; i < tools.length; i++){
+				if(tools[i].needsRebuild())
+					return true;
+			}
+		}
+
+		ITool tools[] = getFilteredTools();
+		for(int i = 0; i < tools.length; i++){
+			if(tools[i].needsRebuild())
+				return true;
+		}
+
+		return false;
 	}
 	
 	/* (non-Javadoc)
@@ -1322,9 +1437,36 @@ public class Configuration extends BuildObject implements IConfiguration {
 	 * @see org.eclipse.cdt.managedbuilder.core.IConfiguration#setRebuildState(boolean)
 	 */
 	public void setRebuildState(boolean rebuild) {
-		rebuildNeeded = rebuild;
-		if(rebuild && !isTemporary())
-			((EnvironmentVariableProvider)ManagedBuildManager.getEnvironmentVariableProvider()).checkBuildPathVariables(this);
+		if(isExtensionElement() && rebuild)
+			return;
+		
+		if(rebuildNeeded != rebuild){
+			rebuildNeeded = rebuild;
+			saveRebuildState();
+		}
+		
+		if(!rebuildNeeded){
+			setResourceChangeState(0);
+			
+			toolChain.setRebuildState(false);
+			
+			if(resourceConfigurationList != null){
+				for(Iterator iter = resourceConfigurationList.iterator();iter.hasNext();){
+					IResourceConfiguration rcCfg = (IResourceConfiguration)iter.next();
+					rcCfg.setRebuildState(false);
+					
+					ITool tools[] = rcCfg.getToolsToInvoke();
+					for(int i = 0; i < tools.length; i++){
+						tools[i].setRebuildState(false);
+					}
+				}
+			}
+	
+			ITool tools[] = getFilteredTools();
+			for(int i = 0; i < tools.length; i++){
+				tools[i].setRebuildState(false);
+			}
+		}
 	}
 
 	/* (non-Javadoc)
@@ -1384,7 +1526,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 		// Send out the event to notify the options that they are about to be removed.
 		// Do not do this for the child resource configurations as they are handled when
 		// the configuration itself is destroyed.
-		ManagedBuildManager.performValueHandlerEvent(this, IManagedOptionValueHandler.EVENT_CLOSE, false);
+//		ManagedBuildManager.performValueHandlerEvent(this, IManagedOptionValueHandler.EVENT_CLOSE, false);
 		// Remove the configurations		
 		for (int i = 0; i < tools.length; i++) {
 			ITool tool = tools[i];
@@ -1398,7 +1540,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 			toolChain.removeOption(opts[j]);
 		}
 		
-		rebuildNeeded = true;
+//		rebuildNeeded = true;
 	}
 
 	/*
@@ -1408,7 +1550,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 	{	
 		String path = file.getFullPath().toString();
 		String resourceName = file.getName();
-		String id = getId() + "." + ManagedBuildManager.getRandomNumber(); //$NON-NLS-1$
+		String id = ManagedBuildManager.calculateChildId(getId(), path);
 		ResourceConfiguration resConfig = new ResourceConfiguration( (IConfiguration) this, id, resourceName, path);
 		
 		//	Get file extension.
@@ -1419,7 +1561,7 @@ public class Configuration extends BuildObject implements IConfiguration {
 		String subId = new String();
 		for (int i = 0; i < tools.length; i++) {
 			if( tools[i].buildsFileType(extString) ) {
-				subId = tools[i].getId() + "." + path; //$NON-NLS-1$
+				subId = ManagedBuildManager.calculateChildId(tools[i].getId(), path);				
 				resConfig.createTool(tools[i], subId, tools[i].getName(), false);
 			}
 		}
@@ -1486,4 +1628,204 @@ public class Configuration extends BuildObject implements IConfiguration {
 			((ResourceConfiguration)iter.next()).updateManagedBuildRevision(revision);
 		}
 	}
+	
+	public void setParent(IConfiguration parent) {
+		if ( this.parent != parent) {
+			this.parent = parent;
+			if (!isExtensionElement())
+				setDirty(true);
+		}		
+	}
+	
+	public ITool calculateTargetTool(){
+		ITool tool = getTargetTool();
+		
+		if(tool == null){
+			tool = getToolFromOutputExtension(getArtifactExtension());
+		}
+		
+		if(tool == null){
+			IConfiguration extCfg;
+			for(extCfg = this; 
+			extCfg != null && !extCfg.isExtensionElement(); 
+			extCfg = extCfg.getParent()){
+			}
+
+			if(extCfg != null){
+				tool = getToolFromOutputExtension(extCfg.getArtifactExtension());
+			}
+		}
+		
+		return tool;
+	}
+	
+	public ITool getToolFromOutputExtension(String extension) {
+		// Treat a null argument as an empty string
+		String ext = extension == null ? EMPTY_STRING : extension;
+		// Get all the tools for the current config
+		ITool[] tools = getFilteredTools();
+		for (int index = 0; index < tools.length; index++) {
+			ITool tool = tools[index];
+			if (tool.producesFileType(ext)) {
+				return tool;
+			}
+		}
+		return null;		
+	}
+	
+	public ITool getToolFromInputExtension(String sourceExtension) {
+		// Get all the tools for the current config
+		ITool[] tools = getFilteredTools();
+		for (int index = 0; index < tools.length; index++) {
+			ITool tool = tools[index];
+			if (tool.buildsFileType(sourceExtension)) {
+				return tool;
+			}
+		}
+		return null;		
+	}
+
+	/*
+	 * The resource delta passed to the builder is not always up-to-date
+	 * for the given configuration because between two builds of the same configuration
+	 * any number of other configuration builds may occur
+	 * that is why we need to keep some information regarding what happened
+	 * with the resource tree between the two configuration builds
+	 *
+	 * The trivial approach implemented currently is to hold
+	 * the general information of whether some resources were 
+	 * removed,changed,etc. and detect whether the rebuild is needed
+	 * based upon this information
+	 * 
+	 * This method adds the resource change state for the configuration
+	 * specifying the resource change type performed on the project
+	 * reported while building another configuration
+	 * The method is not exported to the public API since delta handling
+	 * mechanism will likely to be changed in the future 
+	 *
+	 * In the future we might implement some more smart mechanism
+	 * for tracking delta, e.g calculate the pre-cinfiguration resource delta, etc.
+	 *  
+	 */
+	public void addResourceChangeState(int state){
+		setResourceChangeState(state | resourceChangeState);
+	}
+
+	private void setResourceChangeState(int state){
+		if(resourceChangeState != state){
+			resourceChangeState = state;
+			saveResourceChangeState();
+		}
+	}
+	
+	private boolean resourceChangesRequireRebuild(){
+		return isInternalBuilderEnabled() ?
+				resourceChangeState != 0 :
+					(resourceChangeState & IResourceDelta.REMOVED) == IResourceDelta.REMOVED;
+	}
+	
+	private void saveRebuildState(){
+		PropertyManager.getInstance().setProperty(this, REBUILD_STATE, Boolean.toString(rebuildNeeded));
+	}
+
+	private void saveResourceChangeState(){
+		PropertyManager.getInstance().setProperty(this, RC_CHANGE_STATE, Integer.toString(resourceChangeState));
+	}
+	
+	/*
+	 * Internal Builder state API
+	 * NOTE: this is a temporary API
+	 * In the future we are going present the Internal Builder
+	 * as a special Builder object of the tool-chain and implement the internal
+	 * builder enabling/disabling as the Builder substitution functionality
+	 * 
+	 */
+	
+	/*
+	 * this method is used for enabling/disabling the internal builder
+	 * for the given configuration
+	 * 
+	 * @param enable boolean
+	 */
+	public void enableInternalBuilder(boolean enable){
+//		if(internalBuilderEnabled != enable){
+			internalBuilderEnabled = enable;
+			Preferences prefs = getPreferences(INTERNAL_BUILDER);
+			if(prefs != null){
+				prefs.putBoolean(INTERNAL_BUILDER_ENABLED, enable);
+				try {
+					prefs.flush();
+				} catch (BackingStoreException e) {
+				}
+			}
+//		}
+	}
+	
+	/*
+	 * returns whether the internal builder is enabled
+	 *  
+	 * @return boolean
+	 */
+	public boolean isInternalBuilderEnabled(){
+		Preferences prefs = getPreferences(INTERNAL_BUILDER);
+		
+		return prefs != null ?
+				prefs.getBoolean(INTERNAL_BUILDER_ENABLED, false) : 
+					internalBuilderEnabled;
+	}
+	
+	/*
+	 * 
+	 * sets the Internal Builder mode
+	 * 
+	 * @param ignore if true, internal builder will ignore 
+	 * build errors while building,
+	 * otherwise it will stop at the first build error
+	 */
+	public void setInternalBuilderIgnoreErr(boolean ignore){
+//		if(internalBuilderIgnoreErr != ignore){
+			internalBuilderIgnoreErr = ignore;
+			Preferences prefs = getPreferences(INTERNAL_BUILDER);
+			if(prefs != null){
+				prefs.putBoolean(INTERNAL_BUILDER_IGNORE_ERR, ignore);
+				try {
+					prefs.flush();
+				} catch (BackingStoreException e) {
+				}
+			}
+//		}
+	}
+
+	/*
+	 * returns the Internal Builder mode
+	 * if true, internal builder will ignore build errors while building,
+	 * otherwise it will stop at the first build error
+	 * 
+	 * @return boolean
+	 */
+	public boolean getInternalBuilderIgnoreErr(){
+		Preferences prefs = getPreferences(INTERNAL_BUILDER);
+		
+		return prefs != null ?
+				prefs.getBoolean(INTERNAL_BUILDER_IGNORE_ERR, true) : 
+					internalBuilderIgnoreErr;
+	}
+	
+	private Preferences getPreferences(String name){
+		if(isTemporary)
+			return null;
+		
+		IProject project = (IProject)getOwner();
+		
+		if(project == null || !project.exists() || !project.isOpen())
+			return null;
+
+		Preferences prefs = new ProjectScope(project).getNode(ManagedBuilderCorePlugin.getUniqueIdentifier());
+		if(prefs != null){
+			prefs = prefs.node(getId());
+			if(prefs != null && name != null)
+				prefs = prefs.node(name);
+		}
+		return prefs;
+	}
 }
