@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 Symbian Ltd and others.
+ * Copyright (c) 2005, 2006 Symbian Ltd and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -24,7 +24,6 @@ import org.eclipse.cdt.managedbuilder.core.IOption;
 import org.eclipse.cdt.managedbuilder.core.IOptionCategory;
 import org.eclipse.cdt.managedbuilder.core.ITool;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
-import org.eclipse.cdt.managedbuilder.internal.core.Option;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -63,6 +62,7 @@ public class HoldsOptions extends BuildObject implements IHoldsOptions {
 	private Map optionMap;
 	//  Miscellaneous
 	private boolean isDirty = false;
+	private boolean rebuildState;
 	
 	/*
 	 *  C O N S T R U C T O R S
@@ -190,7 +190,10 @@ public class HoldsOptions extends BuildObject implements IHoldsOptions {
 	public IOption createOption(IOption superClass, String Id, String name, boolean isExtensionElement) {
 		Option option = new Option(this, superClass, Id, name, isExtensionElement);
 		addOption(option);
-		setDirty(true);
+		if(!isExtensionElement){
+			setDirty(true);
+			setRebuildState(true);
+		}
 		return option;
 	}
 	
@@ -214,6 +217,7 @@ public class HoldsOptions extends BuildObject implements IHoldsOptions {
 		getOptionList().remove(option);
 		getOptionMap().remove(option.getId());
 		setDirty(true);
+		setRebuildState(true);
 	}
 	
 	/* (non-Javadoc)
@@ -474,7 +478,8 @@ public class HoldsOptions extends BuildObject implements IHoldsOptions {
 			Iterator iter = optionElements.listIterator();
 			while (iter.hasNext()) {
 				Option option = (Option) iter.next();
-				option.setDirty(false);
+				if(!option.isExtensionElement())
+					option.setDirty(false);
 			}
 		}
 	}
@@ -507,4 +512,100 @@ public class HoldsOptions extends BuildObject implements IHoldsOptions {
 			}
 		}		
 	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IHoldsOptions#getOptionToSet(org.eclipse.cdt.managedbuilder.core.IOption, boolean)
+	 */
+	public IOption getOptionToSet(IOption option, boolean adjustExtension) throws BuildException{
+		IOption setOption = null;
+		if(option.getOptionHolder() != this)
+			option = getOptionBySuperClassId(option.getId());
+		
+		if(adjustExtension){
+			for(; option != null && !option.isExtensionElement(); option=option.getSuperClass()){}
+			
+			if(option != null){
+				IHoldsOptions holder = option.getOptionHolder();
+				if(holder == this)
+					setOption = option;
+				else {
+					IOption newSuperClass = option;
+					if (((Option)option).wasOptRef()) {
+						newSuperClass = option.getSuperClass();
+					}
+					//  Create a new extension Option element
+					String subId;
+					String version = ManagedBuildManager.getVersionFromIdAndVersion(newSuperClass.getId());
+					String baseId = ManagedBuildManager.getIdFromIdAndVersion(newSuperClass.getId());
+					if ( version != null) {
+						subId = baseId + ".adjusted." + new Integer(ManagedBuildManager.getRandomNumber()) + "_" + version; //$NON-NLS-1$ //$NON-NLS-2$ 
+					} else {
+						subId = baseId + ".adjusted." + new Integer(ManagedBuildManager.getRandomNumber()); //$NON-NLS-1$
+					}
+					setOption = createOption(newSuperClass, subId, null, true);
+					((Option)setOption).setAdjusted(true);
+					setOption.setValueType(option.getValueType());
+				}
+			}
+		} else {
+			if(option.getOptionHolder() == this && !option.isExtensionElement()){
+				setOption = option;
+			} else {
+				IOption newSuperClass = option;
+				for(;
+					newSuperClass != null && !newSuperClass.isExtensionElement();
+						newSuperClass = newSuperClass.getSuperClass()){}
+				
+				if (((Option)newSuperClass).wasOptRef()) {
+					newSuperClass = newSuperClass.getSuperClass();
+				}
+				
+				if(((Option)newSuperClass).isAdjustedExtension()){
+					newSuperClass = newSuperClass.getSuperClass();
+				}
+				//  Create an Option element for the managed build project file (.CDTBUILD)
+				String subId;
+				subId = ManagedBuildManager.calculateChildId(newSuperClass.getId(), null);
+				setOption = createOption(newSuperClass, subId, null, false);
+				setOption.setValueType(option.getValueType());
+			}
+		}
+		return setOption;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IHoldsOptions#needsRebuild()
+	 */
+	public boolean needsRebuild() {
+		if(rebuildState)
+			return true;
+		
+		// Otherwise see if any options need saving
+		List optionElements = getOptionList();
+		Iterator iter = optionElements.listIterator();
+		while (iter.hasNext()) {
+			Option option = (Option) iter.next();
+			if (option.needsRebuild()) return true;
+		}
+		
+		return rebuildState;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IHoldsOptions#setRebuildState(boolean)
+	 */
+	public void setRebuildState(boolean rebuild) {
+		rebuildState = rebuild;
+		
+		// Propagate "false" to the children
+		if (!rebuildState) {
+			List optionElements = getOptionList();
+			Iterator iter = optionElements.listIterator();
+			while (iter.hasNext()) {
+				Option option = (Option) iter.next();
+				if(!option.isExtensionElement())
+					option.setRebuildState(false);
+			}
+		}
+	}
 }
