@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 Intel Corporation and others.
+ * Copyright (c) 2005, 2006 Intel Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -25,7 +25,7 @@ import org.eclipse.cdt.managedbuilder.core.IInputOrder;
 import org.eclipse.cdt.managedbuilder.core.IAdditionalInput;
 import org.eclipse.cdt.managedbuilder.core.IManagedConfigElement;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
-import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGenerator;
+import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGeneratorType;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IPath;
@@ -62,11 +62,12 @@ public class InputType extends BuildObject implements IInputType {
 	private Boolean multipleOfType;
 	private Boolean primaryInput;
 	private IConfigurationElement dependencyGeneratorElement = null;
-	private IManagedDependencyGenerator dependencyGenerator = null;
+	private IManagedDependencyGeneratorType dependencyGenerator = null;
 	//  Miscellaneous
 	private boolean isExtensionInputType = false;
 	private boolean isDirty = false;
 	private boolean resolved = true;
+	private boolean rebuildState;
 
 	/*
 	 *  C O N S T R U C T O R S
@@ -131,6 +132,7 @@ public class InputType extends BuildObject implements IInputType {
 			ManagedBuildManager.addExtensionInputType(this);
 		} else {
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -237,6 +239,7 @@ public class InputType extends BuildObject implements IInputType {
 		}
 		
 		setDirty(true);
+		setRebuildState(true);
 	}
 
 	/*
@@ -756,6 +759,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (buildVariable == null || variableName == null || !(variableName.equals(buildVariable))) {
 			buildVariable = variableName;
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -785,6 +789,7 @@ public class InputType extends BuildObject implements IInputType {
 				dependencyContentTypeId = null;
 			}
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -817,6 +822,7 @@ public class InputType extends BuildObject implements IInputType {
 			}
 		}
 		setDirty(true);
+		setRebuildState(true);
 	}
 
 	/* (non-Javadoc)
@@ -826,7 +832,29 @@ public class InputType extends BuildObject implements IInputType {
 		//  Use content type if specified and registered with Eclipse
 		IContentType type = getDependencyContentType();
 		if (type != null) {
-			return ((Tool)tool).getContentTypeFileSpecs(type);
+			String[] exts = ((Tool)tool).getContentTypeFileSpecs(type);
+			//  TODO: This is a temporary hack until we decide how to specify the langauge (C vs. C++)
+			//  of a .h file.  If the content type is the CDT-defined C/C++ content type, then
+			//  add "h" to the list if it is not already there.
+			if (type.getId().compareTo("org.eclipse.cdt.core.cxxHeader") == 0) {  // $NON-NLS-1$
+				boolean h_found = false;
+				for (int i=0; i<exts.length; i++) {
+					if (exts[i].compareTo("h") == 0) {  // $NON-NLS-1$
+						h_found = true;
+						break;
+					}
+				}
+				if (!h_found) {
+					String[] cppexts = new String[exts.length+1];
+					int i = 0;
+					for (; i<exts.length; i++) {
+						cppexts[i] = exts[i];
+					}
+					cppexts[i] = "h";  // $NON-NLS-1$
+					return cppexts;
+				}
+			}
+			return exts;
 		}
 		return getDependencyExtensionsAttribute();
 	}
@@ -852,7 +880,7 @@ public class InputType extends BuildObject implements IInputType {
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.core.build.managed.IInputType#getDependencyGenerator()
 	 */
-	public IManagedDependencyGenerator getDependencyGenerator() {
+	public IManagedDependencyGeneratorType getDependencyGenerator() {
 		if (dependencyGenerator != null) {
 			return dependencyGenerator;
 		}
@@ -860,7 +888,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (element != null) {
 			try {
 				if (element.getAttribute(ITool.DEP_CALC_ID) != null) {
-					dependencyGenerator = (IManagedDependencyGenerator) element.createExecutableExtension(ITool.DEP_CALC_ID);
+					dependencyGenerator = (IManagedDependencyGeneratorType) element.createExecutableExtension(ITool.DEP_CALC_ID);
 					return dependencyGenerator;
 				}
 			} catch (CoreException e) {}
@@ -886,6 +914,7 @@ public class InputType extends BuildObject implements IInputType {
 	public void setDependencyGeneratorElement(IConfigurationElement element) {
 		dependencyGeneratorElement = element;
 		setDirty(true);
+		setRebuildState(true);
 	}
 
 	/* (non-Javadoc)
@@ -909,6 +938,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (multipleOfType == null || !(b == multipleOfType.booleanValue())) {
 			multipleOfType = new Boolean(b);
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -933,6 +963,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (primaryInput == null || !(b == primaryInput.booleanValue())) {
 			primaryInput = new Boolean(b);
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -958,6 +989,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (id == null || optionId == null || !(optionId.equals(id))) {
 			optionId = id;
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -983,6 +1015,7 @@ public class InputType extends BuildObject implements IInputType {
 		if (id == null || assignToOptionId == null || !(assignToOptionId.equals(id))) {
 			assignToOptionId = id;
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -1012,6 +1045,7 @@ public class InputType extends BuildObject implements IInputType {
 				sourceContentTypeId = null;
 			}
 			setDirty(true);
+			setRebuildState(true);
 		}
 	}
 
@@ -1042,6 +1076,7 @@ public class InputType extends BuildObject implements IInputType {
 			}
 		}
 		setDirty(true);
+		setRebuildState(true);
 	}
 
 	/* (non-Javadoc)
@@ -1196,4 +1231,45 @@ public class InputType extends BuildObject implements IInputType {
 	public void setVersion(PluginVersionIdentifier version) {
 		// Do nothing
 	}
+	
+	public boolean needsRebuild(){
+		if(rebuildState)
+			return true;
+		
+		Iterator typeIter = getInputOrderList().iterator();
+		while (typeIter.hasNext()) {
+			InputOrder current = (InputOrder)typeIter.next();
+			if (current.needsRebuild()) return true;
+		}
+		typeIter = getAdditionalInputList().iterator();
+		while (typeIter.hasNext()) {
+			AdditionalInput current = (AdditionalInput)typeIter.next();
+			if (current.needsRebuild()) return true;
+		}
+
+		return rebuildState;
+	}
+	
+	public void setRebuildState(boolean rebuild){
+		if(isExtensionElement() && rebuild)
+			return;
+
+		rebuildState = rebuild;
+		
+		// Propagate "false" to the children
+		if (!rebuild) {
+			Iterator typeIter = getInputOrderList().iterator();
+			while (typeIter.hasNext()) {
+				InputOrder current = (InputOrder)typeIter.next();
+				current.setRebuildState(false);
+			}
+			typeIter = getAdditionalInputList().iterator();
+			while (typeIter.hasNext()) {
+				AdditionalInput current = (AdditionalInput)typeIter.next();
+				current.setRebuildState(false);
+			}
+		}
+
+	}
+
 }
