@@ -7,6 +7,7 @@
  *
  * Contributors:
  *     IBM - Initial API and implementation
+ *     ARM Ltd. - basic tooltip support
  *******************************************************************************/
 package org.eclipse.cdt.managedbuilder.internal.core;
 
@@ -294,6 +295,20 @@ public class OptionReference implements IOption {
 		return option.getCommandFalse();
 	}
 	
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.core.build.managed.IOption#getToolTip()
+	 */
+	public String getToolTip() {
+		return option.getToolTip();
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.core.build.managed.IOption#getContextID()
+	 */
+	public String getContextId() {
+		return option.getContextId();
+	}
+	
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.core.build.managed.IOption#getDefinedSymbols()
 	 */
@@ -750,6 +765,18 @@ public class OptionReference implements IOption {
 	public void setCommandFalse(String cmd) {
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.core.build.managed.IOption#setToolTip(String)
+	 */
+	public void setToolTip(String tooltip) {
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.core.build.managed.IOption#setContextId(String)
+	 */
+	public void setContextId(String contextId) {
+	}
+
 	public PluginVersionIdentifier getVersion() {
 		return option.getVersion();
 	}
