@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -26,6 +26,7 @@ import org.eclipse.cdt.managedbuilder.core.IManagedConfigElement;
 import org.eclipse.cdt.managedbuilder.core.IOption;
 import org.eclipse.cdt.managedbuilder.core.IOptionCategory;
 import org.eclipse.cdt.managedbuilder.core.IOutputType;
+import org.eclipse.cdt.managedbuilder.core.IOptionPathConverter;
 import org.eclipse.cdt.managedbuilder.core.IResourceConfiguration;
 import org.eclipse.cdt.managedbuilder.core.ITool;
 import org.eclipse.cdt.managedbuilder.core.IToolChain;
@@ -33,6 +34,7 @@ import org.eclipse.cdt.managedbuilder.core.IToolReference;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
 import org.eclipse.cdt.managedbuilder.core.IManagedCommandLineGenerator;
 import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGenerator;
+import org.eclipse.cdt.managedbuilder.makegen.IManagedDependencyGeneratorType;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.PluginVersionIdentifier;
@@ -585,6 +587,7 @@ public class ToolReference implements IToolReference {
 	 * @param id
 	 * @return
 	 */
+	/*
 	private OptionReference getOptionReference(String id) {
 		Iterator it = getOptionReferenceList().iterator();
 		while (it.hasNext()) {
@@ -595,6 +598,7 @@ public class ToolReference implements IToolReference {
 		}
 		return null;
 	}
+	*/
 	
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.managedbuilder.core.IToolReference#getOptionReferenceList()
@@ -1115,7 +1119,7 @@ public class ToolReference implements IToolReference {
 		return null;
 	}
 
-	public IManagedDependencyGenerator getDependencyGeneratorForExtension(String sourceExt) {
+	public IManagedDependencyGeneratorType getDependencyGeneratorForExtension(String sourceExt) {
 		return null;
 	}
 
@@ -1244,4 +1248,33 @@ public class ToolReference implements IToolReference {
 	public String getManagedBuildRevision() {
 		return null;
 	}
+	
+	public IOption getOptionToSet(IOption option, boolean adjustExtension){
+		return null;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IHoldsOptions#needsRebuild()
+	 */
+	public boolean needsRebuild() {
+		return false;
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.IHoldsOptions#setRebuildState(boolean)
+	 */
+	public void setRebuildState(boolean rebuild) {
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.cdt.managedbuilder.core.ITool#getPathConverter()
+	 */
+	public IOptionPathConverter getOptionPathConverter() {
+		if (parent!=null)  {
+			return parent.getOptionPathConverter();
+		}
+		return null;
+	}
+	
+	
 }
