@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2002, 2005 IBM Corporation and others.
+ * Copyright (c) 2002, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,10 +18,8 @@ import java.util.ListIterator;
 import java.util.Map;
 
 import org.eclipse.cdt.managedbuilder.core.BuildException;
-import org.eclipse.cdt.managedbuilder.core.IBuildObject;
 import org.eclipse.cdt.managedbuilder.core.IConfiguration;
 import org.eclipse.cdt.managedbuilder.core.IHoldsOptions;
-import org.eclipse.cdt.managedbuilder.core.IManagedOptionValueHandler;
 import org.eclipse.cdt.managedbuilder.core.IOption;
 import org.eclipse.cdt.managedbuilder.core.IOptionCategory;
 import org.eclipse.cdt.managedbuilder.core.IResourceConfiguration;
@@ -39,6 +37,7 @@ import org.eclipse.cdt.managedbuilder.ui.properties.BuildToolSettingsPreferenceS
 import org.eclipse.cdt.managedbuilder.ui.properties.ResourceBuildPropertyPage;
 import org.eclipse.cdt.managedbuilder.ui.properties.ToolListContentProvider;
 import org.eclipse.cdt.managedbuilder.ui.properties.ToolListLabelProvider;
+import org.eclipse.cdt.managedbuilder.ui.properties.ToolListElement;
 import org.eclipse.cdt.ui.dialogs.AbstractCOptionPage;
 import org.eclipse.cdt.ui.dialogs.ICOptionContainer;
 import org.eclipse.cdt.utils.ui.controls.ControlFactory;
@@ -76,8 +75,6 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	//private static final String TREE_LABEL = LABEL + ".ToolTree";	//$NON-NLS-1$
 	//private static final String OPTIONS_LABEL = LABEL + ".ToolOptions";	//$NON-NLS-1$
 	private static final int[] DEFAULT_SASH_WEIGHTS = new int[] { 20, 30 };
-
-	private static final String EMPTY_STRING = new String();
 	
 	/*
 	 * Dialog widgets
@@ -95,10 +92,9 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	private BuildPropertyPage parent;
 	private ResourceBuildPropertyPage resParent;
 	private BuildSettingsPage currentSettingsPage;
-	private IOptionCategory selectedCategory;
+	private ToolListElement selectedElement;
 	private ToolListContentProvider provider;
-	private ITool selectedTool;
-	private Object element;
+	private Object propertyObject;
 	
 	private boolean defaultNeeded;
 
@@ -153,7 +149,7 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		super.setContainer(parent);
 		this.parent = parent;
 		configToPageListMap = new HashMap();
-		this.element = element;
+		this.propertyObject = element;
 		settingsStore = new BuildToolSettingsPreferenceStore(this);
 	}
 
@@ -163,7 +159,7 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		super.setContainer((ICOptionContainer) resParent);
 		this.resParent = resParent;
 		configToPageListMap = new HashMap();
-		this.element = element;
+		this.propertyObject = element;
 		settingsStore = new BuildToolSettingsPreferenceStore(this);
 	}
 	
@@ -215,9 +211,10 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	 * Method displayOptionsForCategory
 	 * @param category
 	 */
-	private void displayOptionsForCategory(IOptionCategory category) {
-		selectedTool = null;
-		selectedCategory = category;
+	private void displayOptionsForCategory(ToolListElement toolListElement) {
+		selectedElement = toolListElement;
+		IOptionCategory category = toolListElement.getOptionCategory();
+		IHoldsOptions optionHolder = toolListElement.getHoldOptions();
 		
 		// Cache the current build setting page
 		BuildSettingsPage oldPage = currentSettingsPage;
@@ -229,18 +226,18 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		while (iter.hasNext()) {
 			BuildSettingsPage page = (BuildSettingsPage) iter.next();
 			if (page instanceof BuildOptionSettingsPage && 
-					((BuildOptionSettingsPage)page).isForCategory(category)) {
+					((BuildOptionSettingsPage)page).isForCategory(optionHolder, category)) {
 				currentSettingsPage = page;
 				break;
 			}
 		}
 		if (currentSettingsPage == null) {
-			if ( this.element instanceof IProject) {
-				currentSettingsPage = new BuildOptionSettingsPage(parent,parent.getSelectedConfigurationClone(), category);
+			if ( this.propertyObject instanceof IProject) {
+				currentSettingsPage = new BuildOptionSettingsPage(parent,parent.getSelectedConfigurationClone(), optionHolder, category);
 				pages.add(currentSettingsPage);
 				currentSettingsPage.setContainer(parent);
-			} else if ( this.element instanceof IFile) {
-				currentSettingsPage = new BuildOptionSettingsPage(resParent,resParent.getCurrentResourceConfigClone(), category);
+			} else if ( this.propertyObject instanceof IFile) {
+				currentSettingsPage = new BuildOptionSettingsPage(resParent,resParent.getCurrentResourceConfigClone(), optionHolder, category);
 				pages.add(currentSettingsPage);
 				currentSettingsPage.setContainer(resParent);
 			}
@@ -274,11 +271,9 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	 * Method displayOptionsForTool
 	 * @param tool
 	 */
-	private void displayOptionsForTool(ITool tool) {
-		// Unselect the category
-		selectedCategory = null;
-		// record that the tool selection has changed
-		selectedTool = tool;
+	private void displayOptionsForTool(ToolListElement toolListElement) {
+		selectedElement = toolListElement;
+		ITool tool = toolListElement.getTool();
 		
 		// Cache the current build setting page
 		BuildSettingsPage oldPage = currentSettingsPage;
@@ -296,12 +291,12 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 			}
 		}
 		if (currentSettingsPage == null) {
-			if ( this.element instanceof IProject) {
+			if ( this.propertyObject instanceof IProject) {
 				currentSettingsPage = new BuildToolSettingsPage(parent, 
 						parent.getSelectedConfigurationClone(), tool);
 				pages.add(currentSettingsPage);
 				currentSettingsPage.setContainer(parent);
-			} else if(this.element instanceof IFile) {
+			} else if(this.propertyObject instanceof IFile) {
 				currentSettingsPage = new BuildToolSettingsPage(resParent,
 						resParent.getCurrentResourceConfigClone(), tool);
 				pages.add(currentSettingsPage);
@@ -379,101 +374,112 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 
 	public void setVisible(boolean visible){
 		if(visible){
-			selectedCategory = null;
-			selectedTool = null;
+			selectedElement = null;
 			handleOptionSelection();
 		}
 		super.setVisible(visible);
 	}
 
 	protected void setValues() {
-		
-		IConfiguration config = null;	
-		IResourceConfiguration resConfig = null;
-		
+		/*
+		 *  This method updates the context of the build property pages
+		 *   - Which configuration/resource configuration is selected
+		 *   - Which tool/option category is selected
+		 *    
+		 *  It is called:
+		 *   - When a property page becomes visible
+		 *   - When the user changes the configuration selection
+		 *   - When the user changes the "exclude" setting for a resource
+		 */
+
+		//  Create the Tree Viewer content provider if first time
 		if (provider == null) {
-//			IResource element = parent.getProject(); 
-			IResource resource = (IResource) element;
+			IResource resource = (IResource) propertyObject;
 			provider = new ToolListContentProvider(resource.getType());
 			optionList.setContentProvider(provider);
 		}
-		if ( element instanceof IProject ) {
+
+		//  Update the selected configuration and the Tree Viewer
+		IConfiguration config = null;	
+		IResourceConfiguration resConfig = null;
+		ToolListElement[] newElements;
+		
+		if ( propertyObject instanceof IProject ) {
 			config = parent.getSelectedConfigurationClone();	
 			optionList.setInput(config);	
-		} else if ( element instanceof IFile){
+			newElements = (ToolListElement[])provider.getElements(config);
+		} else if ( propertyObject instanceof IFile){
 			resConfig = resParent.getCurrentResourceConfigClone();
 			optionList.setInput(resConfig);
-		}
+			newElements = (ToolListElement[])provider.getElements(resConfig);
+		} else
+			return;		// This should not happen
 		
 		optionList.expandAll();
 		
-		// Determine what the selection in the tree should be
-		Object primary = null;
-		if (selectedTool != null) {
-			// There is a selected tool defined.  See if it matches any current tool (by name)
-			ITool[] tools = null;
-			if ( element instanceof IProject ) {
-				tools = config.getFilteredTools();
-			} else if ( element instanceof IFile){
-				tools = resConfig.getTools();
-			}			
-			String matchName = selectedTool.getName();
-			for (int i=0; i<tools.length; i++) {
-				ITool tool = tools[i];
-				if (tool.getName().equals(matchName)) {
-					primary = tool;
-					break;
-				}
-			}
-		} else if (selectedCategory != null) {
-			// There is a selected option or category.  
-			// See if it matches any category in the current config (by name)
-			ITool[] tools = null;
-			IToolChain toolChain = null;
-			
-			if ( element instanceof IProject ) {
-				tools = config.getFilteredTools();
-				toolChain = config.getToolChain();
-			} else if ( element instanceof IFile){
-				tools = resConfig.getTools();
-			}
-			IBuildObject catOrTool = selectedCategory;
-			// Make the match name
-			String matchName = makeMatchName(catOrTool);
-			// Search for selected category/tool in toolChain
-			if ( toolChain != null ) {
-				primary = findOptionCategoryByMatchName(matchName, toolChain.getChildCategories());
-			}			
-			// Search for selected category/tool in tools
-			if ( primary == null ) {
-				for (int i=0; i<tools.length && primary == null; i++) {
-					primary = findOptionCategoryByMatchName(matchName, tools[i].getChildCategories());
-				}
-			}
-		} 
-		
-		if (primary == null) {
-			// Select the first tool in the list
-			Object[] elements = null;
-			if( element instanceof IProject){
-				elements = provider.getElements(parent.getSelectedConfigurationClone());
-			} else if ( element instanceof IFile) {
-				elements = provider.getElements(resParent.getCurrentResourceConfigClone());
-			}
-			primary = elements.length > 0 ? elements[0] : null;			
+		//  Determine what the selection in the tree should be
+		//  If the saved selection is not null, try to match the saved selection
+		//  with an object in the new element list.
+		//  Otherwise, select the first tool in the tree
+		Object primaryObject = null;
+		if (selectedElement != null) {
+			selectedElement = matchSelectionElement(selectedElement, newElements);
 		}
-		
-		if (primary != null) {
-			if(primary instanceof IOptionCategory){
-				if(resConfig != null)
-					settingsStore.setSelection(resConfig,(IOptionCategory)primary);
-				else
-					settingsStore.setSelection(config,(IOptionCategory)primary);
+			
+		if (selectedElement == null) {
+			selectedElement = (ToolListElement)(newElements.length > 0 ? newElements[0] : null);
+		}
+			
+		if (selectedElement != null) {
+			primaryObject = selectedElement.getTool();
+			if (primaryObject == null) {
+				primaryObject = selectedElement.getOptionCategory();
+			}
+			if (primaryObject != null) {
+				if (primaryObject instanceof IOptionCategory) {
+					if(resConfig != null)
+						settingsStore.setSelection(resConfig, selectedElement, (IOptionCategory)primaryObject);
+					else
+						settingsStore.setSelection(config, selectedElement, (IOptionCategory)primaryObject);
+				}
+				optionList.setSelection(new StructuredSelection(selectedElement), true);
 			}
-			optionList.setSelection(new StructuredSelection(primary), true);
 		}
 	}
+						
+	private ToolListElement matchSelectionElement(ToolListElement currentElement, ToolListElement[] elements) {
+		//  First, look for an exact match
+		ToolListElement match = exactMatchSelectionElement(currentElement, elements);
+		if (match == null)
+			//  Else, look for the same tool/category in the new set of elements
+			match = equivalentMatchSelectionElement(currentElement, elements);
+		return match;
+	}
 
+	private ToolListElement exactMatchSelectionElement(ToolListElement currentElement, ToolListElement[] elements) {
+		for (int i=0; i<elements.length; i++) {
+			ToolListElement e = elements[i];
+			if (e == currentElement) {
+				return currentElement;
+			}
+			e = exactMatchSelectionElement(currentElement, e.getChildElements());
+			if (e != null) return e;
+		}
+		return null;
+	}
+
+	private ToolListElement equivalentMatchSelectionElement(ToolListElement currentElement, ToolListElement[] elements) {
+		for (int i=0; i<elements.length; i++) {
+			ToolListElement e = elements[i];
+			if (e.isEquivalentTo(currentElement)) {
+				return e;
+			}
+			e = equivalentMatchSelectionElement(currentElement, e.getChildElements());
+			if (e != null) return e;
+		}
+		return null;
+	}
+	
 	public void removeValues(String id) {
 	}
 
@@ -482,17 +488,24 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		IStructuredSelection selection = (IStructuredSelection) optionList.getSelection();
 	
 		// Set the option page based on the selection
-		Object element = selection.getFirstElement();
-		if(element instanceof IOptionCategory){
-			if(resParent != null)
-				settingsStore.setSelection(resParent.getCurrentResourceConfigClone(),(IOptionCategory)element);
-			else
-				settingsStore.setSelection(parent.getSelectedConfigurationClone(),(IOptionCategory)element);
-		}
-		if (element instanceof ITool) {
-			displayOptionsForTool((ITool)element);
-		} else if (element instanceof IOptionCategory) {
-			displayOptionsForCategory((IOptionCategory)element);
+		ToolListElement toolListElement = (ToolListElement)selection.getFirstElement();
+		if (toolListElement != null) {
+			IOptionCategory cat = toolListElement.getOptionCategory();
+			if (cat == null)
+				cat = (IOptionCategory)toolListElement.getTool();
+			if (cat != null) {
+				if(resParent != null)
+					settingsStore.setSelection(resParent.getCurrentResourceConfigClone(), toolListElement, cat);
+				else
+					settingsStore.setSelection(parent.getSelectedConfigurationClone(), toolListElement, cat);
+			}
+
+			cat = toolListElement.getOptionCategory();
+			if (cat != null) {
+				displayOptionsForCategory(toolListElement);
+			} else {
+				displayOptionsForTool(toolListElement);
+			}
 		}
 	}
 
@@ -500,7 +513,7 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	 * Call an MBS CallBack function to inform that default settings have been applied.
 	 * This has to be sent to all the Options associated with this configuration.
 	 */
-	private void performSetDefaultsEventCallBack() {
+/*	private void performSetDefaultsEventCallBack() {
 		
 		if ( element instanceof IProject) {
 			// Do not send the event to the child resource configurations, as performDefaults
@@ -514,16 +527,16 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 					IManagedOptionValueHandler.EVENT_SETDEFAULT);
 		}
 	}
-	
+*/	
 	/*
 	 *  (non-Javadoc)
 	 * @see org.eclipse.cdt.ui.dialogs.ICOptionPage#performDefaults()
 	 */
 	public void performDefaults() {
-		if ( element instanceof IProject) {
-			performDefaults( (IProject)element);
-		} else if ( element instanceof IFile) {
-			performDefaults( (IFile)element);
+		if ( propertyObject instanceof IProject) {
+			performDefaults( (IProject)propertyObject);
+		} else if ( propertyObject instanceof IFile) {
+			performDefaults( (IFile)propertyObject);
 		}
 		defaultNeeded = true;
 		return;
@@ -538,13 +551,14 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		ManagedBuildManager.resetConfiguration(parent.getProject(), parent.getSelectedConfigurationClone());
 		ITool tools[] = parent.getSelectedConfigurationClone().getFilteredTools();
 		for( int i = 0; i < tools.length; i++ ){
-			if(!tools[i].getCustomBuildStep())
+			if(!tools[i].getCustomBuildStep()) {
 				tools[i].setToolCommand(null);
+				tools[i].setCommandLinePattern(null);
+			}
 		}
 		
 		// Reset the category or tool selection and run selection event handler
-		selectedCategory = null;
-		selectedTool = null;
+		selectedElement = null;
 		handleOptionSelection();
 		
 		setDirty(true);
@@ -558,13 +572,14 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		ManagedBuildManager.resetResourceConfiguration(resParent.getProject(), resParent.getCurrentResourceConfigClone());
 		ITool tools[] = resParent.getCurrentResourceConfigClone().getTools();
 		for( int i = 0; i < tools.length; i++ ){
-			if(!tools[i].getCustomBuildStep())
+			if(!tools[i].getCustomBuildStep()) {
 				tools[i].setToolCommand(null);
+				tools[i].setCommandLinePattern(null);
+			}
 		}
 
 		// Reset the category or tool selection and run selection event handler
-		selectedCategory = null;
-		selectedTool = null;
+		selectedElement = null;
 		handleOptionSelection();
 		
 		setDirty(true);
@@ -576,16 +591,16 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	 */
 	public void performApply(IProgressMonitor monitor) throws CoreException {
 		
-		if(element instanceof IFile)
+		if(propertyObject instanceof IFile)
 			resParent.getCurrentResourceConfig(true);
 		
 		if(defaultNeeded){
-			if(element instanceof IFile)
+			if(propertyObject instanceof IFile)
 				ManagedBuildManager.resetResourceConfiguration(resParent.getProject(), resParent.getCurrentResourceConfig(true));
 			else
 				ManagedBuildManager.resetConfiguration(parent.getProject(), parent.getSelectedConfiguration());
 
-			performSetDefaultsEventCallBack();
+//			performSetDefaultsEventCallBack();
 			
 			defaultNeeded = false;
 		}
@@ -622,33 +637,58 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		IHoldsOptions realHo = page.getRealHoldsOptions(holder);
 		
 		if(realHo != null){
-			if(holder instanceof ITool)
-				((ITool)realHo).setToolCommand(((ITool)holder).getToolCommand());
-			
+			IHoldsOptions realHolder;
+			IConfiguration realCfg = null;
+			IResourceConfiguration realRcCfg = null;
+			if(resParent != null){ 
+				realHolder = resParent.getRealHoldsOptions(holder);
+				realRcCfg = (IResourceConfiguration)((ITool)realHolder).getParent();
+				realCfg = realRcCfg.getParent();
+			} else {
+				realHolder = parent.getRealHoldsOptions(holder);
+				realCfg = parent.getConfigurationFromHoldsOptions(realHolder);
+			}		
+			if(holder instanceof ITool) {
+				String currentValue = ((ITool)realHo).getToolCommand();
+				if (!(currentValue.equals(((ITool)holder).getToolCommand()))) {
+					((ITool)realHo).setToolCommand(((ITool)holder).getToolCommand());
+					//if(resParent != null){
+					    // TODO: This causes the entire project to be rebuilt.  Is there a way to only have this 
+					    //       file rebuilt?  "Clean" its output?  Change its modification date?
+						//realCfg.setRebuildState(true);
+					//} else {
+//						realCfg.setRebuildState(true);
+					//}
+				}
+				currentValue = ((ITool)realHo).getCommandLinePattern();
+				if (!(currentValue.equals(((ITool)holder).getCommandLinePattern()))) {
+					((ITool)realHo).setCommandLinePattern(((ITool)holder).getCommandLinePattern());
+					//if(resParent != null){
+					    // TODO: This causes the entire project to be rebuilt.  Is there a way to only have this 
+					    //       file rebuilt?  "Clean" its output?  Change its modification date?
+						//realCfg.setRebuildState(true);
+					//} else {
+//						realCfg.setRebuildState(true);
+					//}
+				}
+			}
 			IOption options[] = holder.getOptions();
 			for(int i = 0; i < options.length; i++){
-				saveOption(options[i], holder);
+				saveOption(options[i], holder, realHolder, realCfg, realRcCfg);
 			}
 		}
 	}
-	private void saveOption(IOption clonedOption, IHoldsOptions cloneHolder){
-		IConfiguration realCfg = null;
-		IResourceConfiguration realRcCfg = null;
-		IBuildObject handler = null;
+	private void saveOption(IOption clonedOption, IHoldsOptions cloneHolder,
+			IHoldsOptions realHolder, IConfiguration realCfg, IResourceConfiguration realRcCfg){
 		IOption realOption;
-		IHoldsOptions realHolder;
+//		IBuildObject handler = null;
 		
 		if(resParent != null){ 
 			realOption = resParent.getRealOption(clonedOption,cloneHolder);
-			realHolder = resParent.getRealHoldsOptions(cloneHolder);
-			realRcCfg = (IResourceConfiguration)((ITool)realHolder).getParent();
-			realCfg = realRcCfg.getParent();
-			handler = realRcCfg;
+//			handler = realRcCfg;
 		} else {
 			realOption = parent.getRealOption(clonedOption,cloneHolder);
-			realHolder = parent.getRealHoldsOptions(cloneHolder);
-			realCfg = parent.getConfigurationFromHoldsOptions(realHolder);
-			handler = realCfg;
+//			handler = realCfg;
 		}		
 		
 		try {
@@ -729,7 +769,7 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 			if (setOption == null)
 				setOption = realOption;
 			
-			if (setOption.getValueHandler().handleValue(
+/*			if (setOption.getValueHandler().handleValue(
 					handler, 
 					setOption.getOptionHolder(), 
 					setOption,
@@ -739,7 +779,8 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 				// May need to do something here say log a message.
 			} else {
 				// Event handling Failed. 
-			} 
+			}
+*/ 
 		} catch (BuildException e) {
 		} catch (ClassCastException e) {
 		}
@@ -752,7 +793,7 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		IToolChain tc = cfg.getToolChain();
 		saveHoldsOptions(tc);
 		
-		ITool tools[] = tc.getTools();
+		ITool tools[] = cfg.getFilteredTools();
 		for(int i = 0; i < tools.length; i++){
 			saveHoldsOptions(tools[i]);
 		}
@@ -770,6 +811,13 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		for(int i = 0; i < tools.length; i++){
 			ITool tool = tools[i];
 			if(!tool.getCustomBuildStep()){
+				ITool cfgTool = cfg.getToolChain().getTool(tool.getSuperClass().getId());
+				//  Check for a non-default command or command-line-pattern
+				if(cfgTool != null){
+					if (!(tool.getToolCommand().equals(cfgTool.getToolCommand()))) return false;
+					if (!(tool.getCommandLinePattern().equals(cfgTool.getCommandLinePattern()))) return false;
+				}
+				//  Check for a non-default option
 				IOption options[] = tool.getOptions();
 				for( int j = 0; j < options.length; j++){
 					IOption option = options[j];
@@ -781,7 +829,6 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 						} while((ext = ext.getSuperClass()) != null);
 
 						if(ext != null){
-							ITool cfgTool = cfg.getToolChain().getTool(tool.getSuperClass().getId());
 							if(cfgTool != null){
 								IOption defaultOpt = cfgTool.getOptionBySuperClassId(ext.getId());
 								try {								
@@ -811,13 +858,13 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 	 */
 	private List getPagesForConfig() {
 		List pages = null;
-		if ( element instanceof IProject) {
+		if ( propertyObject instanceof IProject) {
 //			 Make sure that something was selected
 			if (parent.getSelectedConfigurationClone() == null) {
 				return null;
 			}
 			pages = (List) configToPageListMap.get(parent.getSelectedConfigurationClone().getId());	
-		} else if (element instanceof IFile) {
+		} else if (propertyObject instanceof IFile) {
 			if ( resParent.getCurrentResourceConfigClone() == null ) {
 				return null;
 			}
@@ -826,9 +873,9 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		
 		if (pages == null) {
 			pages = new ArrayList();
-			if ( element instanceof IProject) {
+			if ( propertyObject instanceof IProject) {
 				configToPageListMap.put(parent.getSelectedConfigurationClone().getId(), pages);	
-			} else if ( element instanceof IFile) {
+			} else if ( propertyObject instanceof IFile) {
 				configToPageListMap.put(resParent.getCurrentResourceConfigClone().getId(), pages);
 			}
 		}
@@ -903,59 +950,5 @@ public class ToolsSettingsBlock extends AbstractCOptionPage {
 		}
 		return (BuildMacroProvider)ManagedBuildManager.getBuildMacroProvider();
 	}
-	
-	/**
-	 * Creates a name that uniquely identifies a category. The match name is 
-	 * a concatenation of the tool and categories, e.g. Tool->Cat1->Cat2 
-	 * maps onto the string "Tool|Cat1|Cat2|"
-	 * 
-	 * @param category or tool for which to build the match name 
-	 * @return match name
-	 */
-	private String makeMatchName(IBuildObject catOrTool) {
-		String catName = EMPTY_STRING;
-		
-		// Build the match name.
-		do {
-			catName = catOrTool.getName() + "|" + catName; //$NON-NLS-1$
-			if (catOrTool instanceof ITool) break;
-			else if (catOrTool instanceof IOptionCategory) {
-				catOrTool = ((IOptionCategory)catOrTool).getOwner();					
-			} else 
-				break;
-		} while (catOrTool != null);
-		
-		return catName;
-	}
-	
-	/**
-	 * Finds an option category from an array of categories by comparing against
-	 * a match name. The match name is a concatenation of the tool and categories, 
-	 * e.g. Tool->Cat1->Cat2 maps onto the string "Tool|Cat1|Cat2|"
-	 * 
-	 * @param matchName an identifier to search 
-	 * @param categories as returned by getChildCategories(), i.e. non-flattened 
-	 * @return category or tool, if found and null otherwise
-	 */
-	private Object findOptionCategoryByMatchName(String matchName, IOptionCategory[] cats) {
-		Object primary = null;
-		
-		for (int j=0; j<cats.length; j++) {
-			IBuildObject catOrTool = cats[j]; 
-			// Build the match name
-			String catName = makeMatchName(catOrTool);
-			// Check whether the name matches
-			if (catName.equals(matchName)) {
-				primary = cats[j];
-				break;
-			} else if (matchName.startsWith(catName)) {
-				// If there is a common root then check for any further children
-				primary = findOptionCategoryByMatchName(matchName, cats[j].getChildCategories());
-				if (primary != null)
-					break;
-			}
-		}
-		return primary;
-	}	
 }
 
