@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2002, 2005 Rational Software Corporation and others.
+ * Copyright (c) 2002, 2006 Rational Software Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,8 +11,10 @@
 package org.eclipse.cdt.managedbuilder.ui.properties;
 
 import java.lang.reflect.InvocationTargetException;
+import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.ListIterator;
 
 import org.eclipse.cdt.managedbuilder.core.IConfiguration;
 import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
@@ -56,7 +58,7 @@ import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IWorkbenchPropertyPage;
 import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 
 public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWorkbenchPropertyPage, 
 					IPreferencePageContainer, ICOptionContainer {
@@ -74,9 +76,8 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 	private static final String TIP = PREFIX + ".tip";	//$NON-NLS-1$
 	private static final String PLAT_TIP = TIP + ".platform";	//$NON-NLS-1$
 	private static final String CONF_TIP = TIP + ".config";	//$NON-NLS-1$
-	private static final String ADD_TIP = TIP + ".addconf";	//$NON-NLS-1$
+	private static final String MANAGE_TIP = TIP + ".manageconf";	//$NON-NLS-1$
 	private static final String MANAGE_TITLE = PREFIX + ".manage.title";	//$NON-NLS-1$
-	private static final String ID_SEPARATOR = ".";	//$NON-NLS-1$
 	private static final String MSG_CLOSEDPROJECT = "MngMakeProjectPropertyPage.closedproject"; //$NON-NLS-1$
 	
 	private static final String MSG_UNSUPPORTED_PROJ = PREFIX + ".unsupported.proj"; //$NON-NLS-1$
@@ -137,6 +138,7 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 			contentForClosedProject(composite);
 		} else {
 			contentForCProject(composite);
+			PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, ManagedBuilderHelpContextIds.MAN_PROJ_BUILD_PROP);
 		}
 
 		return composite;
@@ -152,7 +154,7 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 			noContent(parent,ManagedBuilderUIMessages.getResourceString("BuildPropertyPage.error.version_low"));	//$NON-NLS-1$
 			return;
 		}
-		projectTypes = ManagedBuildManager.getDefinedProjectTypes();
+		projectTypes = getProjectTypes();
 		IProjectType defaultProjectType = info.getManagedProject().getProjectType();
 
 		// Add a config selection area
@@ -185,7 +187,7 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 		});
 		configSelector.setToolTipText(ManagedBuilderUIMessages.getResourceString(CONF_TIP));
 		manageConfigs = ControlFactory.createPushButton(configGroup, ManagedBuilderUIMessages.getResourceString(ADD_CONF));
-		manageConfigs.setToolTipText(ManagedBuilderUIMessages.getResourceString(ADD_TIP));
+		manageConfigs.setToolTipText(ManagedBuilderUIMessages.getResourceString(MANAGE_TIP));
 		manageConfigs.addSelectionListener(new SelectionAdapter() {
 		public void widgetSelected(SelectionEvent e) {
 				handleManageConfig();
@@ -225,8 +227,35 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 
 		// Do not call this until the widgets are constructed		
 		handleProjectTypeSelection();
+	}
+
+	private IProjectType[] getProjectTypes() {
+		
+		IProjectType [] allProjectTypes = ManagedBuildManager.getDefinedProjectTypes();
+		ArrayList types = new ArrayList();
+		
+		// The projectTypes that has 'convertToId' attribute is used only in conversion process,
+		// These projectTypes should not be used during project creation or during the display of projectTypes
+		// to the user. So we need to filter the projectTypes based on the 'convertToId' attribute.
+		// Check if project type has 'convertToId' attribute. If so, then do not add it to the list.
+		
+		
+		for (int i = 0; i < allProjectTypes.length; i++) {
+			IProjectType projectType = allProjectTypes[i];
+			if (!projectType.getConvertToId().equals(""))
+				continue;
+			types.add(projectType);
+		}
+		types.trimToSize();
+		
+		IProjectType [] tmpProjectTypes = new IProjectType[types.size()];
+		ListIterator iter = types.listIterator();
+		int index = 0;
+		while (iter.hasNext()) {
+			tmpProjectTypes[index++] = (IProjectType) iter.next();
+		}
+		return tmpProjectTypes;
 		
-		WorkbenchHelp.setHelp(parent, ManagedBuilderHelpContextIds.MAN_PROJ_BUILD_PROP);
 	}
 
 	private void contentForClosedProject(Composite parent) {
@@ -309,12 +338,12 @@ public class BuildPropertyPage extends AbstractBuildPropertyPage implements IWor
 	 */
 	protected boolean writeBuildInfo() {
 		// Write out the build model info
-		ManagedBuildManager.saveBuildInfo(getProject(), false);
+		boolean saveBuildStatus = ManagedBuildManager.saveBuildInfo(getProject(), false);
 		IManagedBuildInfo bi = ManagedBuildManager.getBuildInfo(getProject());
 		if (bi != null & bi instanceof ManagedBuildInfo) {
 			((ManagedBuildInfo)bi).initializePathEntries();
 		}
-		return true;
+		return saveBuildStatus;
 	}
 	
 	/*
