@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,6 +13,7 @@ package org.eclipse.cdt.managedbuilder.ui.properties;
 import org.eclipse.cdt.managedbuilder.core.IManagedBuildInfo;
 import org.eclipse.cdt.managedbuilder.core.IProjectType;
 import org.eclipse.cdt.managedbuilder.core.IManagedProject;
+import org.eclipse.cdt.managedbuilder.core.IToolChain;
 import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
 
 import java.util.ArrayList;
@@ -54,8 +55,6 @@ public class NewConfigurationDialog extends StatusDialog {
 	private static final String INVALID = ERROR + ".invalidName";	//$NON-NLS-1$	
 	private static final String DESCRIPTION = LABEL + ".description";	//$NON-NLS-1$
 	
-	private static final String ID_SEPARATOR = ".";	//$NON-NLS-1$
-
 	// Widgets
 	private Button btnClone;
 	private Button btnCopy;
@@ -348,11 +347,19 @@ public class NewConfigurationDialog extends StatusDialog {
 			defaultConfigs = cfgs;
 		else {
 			ArrayList list = new ArrayList();
-			for (int i = 0; i < cfgs.length; i++){
-				if(cfgs[i].isSupported())
+			for (int i = 0; i < cfgs.length; i++) {
+				if (cfgs[i].isSupported()) {
+					IToolChain tc = cfgs[i].getToolChain();
+
+					// Determine if the tool-chain has 'convertToId' attribute.
+					// If so, do not add this configuration to the list.
+					if (!tc.getConvertToId().equals(""))
+						continue;
 					list.add(cfgs[i]);
+				}
 			}
-			defaultConfigs = (IConfiguration[])list.toArray(new IConfiguration[list.size()]);
+			defaultConfigs = (IConfiguration[]) list
+					.toArray(new IConfiguration[list.size()]);
 		}
 	
 		if(defaultConfigs.length != 0){
@@ -540,23 +547,19 @@ public class NewConfigurationDialog extends StatusDialog {
 	 * the dialog.
 	 */
 	public IConfiguration newConfiguration(IManagedBuildInfo info) {
-		int id = ManagedBuildManager.getRandomNumber();
 		
-		// Create ID for the new component based on the parent ID and random component
-		String newId = parentConfig.getId();
-		int index = newId.lastIndexOf(ID_SEPARATOR);
-		if (index > 0) {
-			String lastComponent = newId.substring(index + 1, newId.length());
-			if (Character.isDigit(lastComponent.charAt(0))) {
-				// Strip the last component
-				newId = newId.substring(0, index);
-			}
-		}
-		newId += ID_SEPARATOR + id;
+		String newId = null;
 		IConfiguration newConfig;
+				
 		if (parentConfig.isExtensionElement()) {
+			// If parent config is an extension element,
+			// Create ID for the new component based on the parentConfig's id and random component
+			newId = ManagedBuildManager.calculateChildId(parentConfig.getId(), null);
 			newConfig = info.getManagedProject().createConfiguration(parentConfig, newId);
 		} else {
+			// If parent config is not an extension element, then
+			// Create ID for the new component based on the parentConfig's parent id and random component
+			newId = ManagedBuildManager.calculateChildId(parentConfig.getParent().getId(), null);
 			newConfig = info.getManagedProject().createConfigurationClone(parentConfig, newId);
 		}
 		
