@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 Intel Corporation and others.
+ * Copyright (c) 2004, 2006 Intel Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -58,7 +58,7 @@ import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IWorkbenchPropertyPage;
 import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 
 
 public class ResourceBuildPropertyPage extends AbstractBuildPropertyPage implements
@@ -151,6 +151,8 @@ public class ResourceBuildPropertyPage extends AbstractBuildPropertyPage impleme
 		    noContent(composite,ManagedBuilderUIMessages.getResourceString(NOTMBSFILE_LABEL));
 		}
 		
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(getControl(), ManagedBuilderHelpContextIds.MAN_PROJ_BUILD_PROP);
+		
 		return composite;
 	}
 	
@@ -227,8 +229,6 @@ public class ResourceBuildPropertyPage extends AbstractBuildPropertyPage impleme
 		//	Update the contents of the configuration widget
 		populateConfigurations();
 		fOptionBlock.createContents(tabGroup, getElement());
-
-		WorkbenchHelp.setHelp(composite,ManagedBuilderHelpContextIds.MAN_PROJ_BUILD_PROP);
 	}
 	
 	protected void noContent(Composite composite, String message) {
@@ -452,44 +452,46 @@ public class ResourceBuildPropertyPage extends AbstractBuildPropertyPage impleme
 	 */
 	public boolean performOk() {
 		
+		boolean retStatus = true;
+		
 		//	If there is no content on the page, then there is nothing to do
 		if (noContentOnPage) return true;
 
 		//	If the user did not visit this page, then there is nothing to do.
 		if (!displayedConfig) return true;
 
-			IRunnableWithProgress runnable = new IRunnableWithProgress() {
-				public void run(IProgressMonitor monitor) {
-					if(containsDefaults()){
-						removeCurrentResourceConfig();
-						return;
-					}
-					
-					fOptionBlock.performApply(monitor);
-					getCurrentResourceConfig(true).setExclude(getCurrentResourceConfigClone().isExcluded());
+		IRunnableWithProgress runnable = new IRunnableWithProgress() {
+			public void run(IProgressMonitor monitor) {
+				if(containsDefaults()){
+					removeCurrentResourceConfig();
+					return;
 				}
-			};
-			IRunnableWithProgress op = new WorkspaceModifyDelegatingOperation(runnable);
-			try {
-				new ProgressMonitorDialog(getShell()).run(false, true, op);
-			} catch (InvocationTargetException e) {
-				Throwable e1 = e.getTargetException();
-				ManagedBuilderUIPlugin.errorDialog(getShell(), ManagedBuilderUIMessages.getResourceString("ManagedProjectPropertyPage.internalError"),e1.toString(), e1); //$NON-NLS-1$
-				return false;
-			} catch (InterruptedException e) {
-				// cancelled
-				return false;
+				
+				fOptionBlock.performApply(monitor);
+				getCurrentResourceConfig(true).setExclude(getCurrentResourceConfigClone().isExcluded());
 			}
-	
-			// Write out the build model info
+		};
+		IRunnableWithProgress op = new WorkspaceModifyDelegatingOperation(runnable);
+		try {
+			new ProgressMonitorDialog(getShell()).run(false, true, op);
+		} catch (InvocationTargetException e) {
+			Throwable e1 = e.getTargetException();
+			ManagedBuilderUIPlugin.errorDialog(getShell(), ManagedBuilderUIMessages.getResourceString("ManagedProjectPropertyPage.internalError"),e1.toString(), e1); //$NON-NLS-1$
+			return false;
+		} catch (InterruptedException e) {
+			// cancelled
+			return false;
+		}
+
+		// Write out the build model info
 		ManagedBuildManager.setDefaultConfiguration(getProject(), getSelectedConfiguration());
 		
-		if(getCurrentResourceConfigClone().isDirty()){
-			selectedConfiguration.setRebuildState(true);
+		if (getCurrentResourceConfigClone().isDirty()) {
+//			selectedConfiguration.setRebuildState(true);
 			getCurrentResourceConfigClone().setDirty(false);
 		}
 
-		ManagedBuildManager.saveBuildInfo(getProject(), false);
+		retStatus = ManagedBuildManager.saveBuildInfo(getProject(), false);
 		
 		IManagedBuildInfo bi = ManagedBuildManager.getBuildInfo(getProject());
 		if (bi != null & bi instanceof ManagedBuildInfo) {
@@ -498,10 +500,11 @@ public class ResourceBuildPropertyPage extends AbstractBuildPropertyPage impleme
 		
 		EnvironmentVariableProvider.fUserSupplier.checkInexistentConfigurations(clonedConfiguration.getManagedProject());
 
-		return true;
+		return retStatus;
 	}
 	
 	public boolean containsDefaults(){
+		//  Check for a non-default "excluded" value
 		if(getCurrentResourceConfigClone().isExcluded() != DEFAULT_EXCLUDE_VALUE)
 			return false;
 		return fOptionBlock.containsDefaults();
