@@ -103,6 +103,7 @@ import org.eclipse.ui.model.IWorkbenchAdapter;
 import org.eclipse.ui.part.ISetSelectionTarget;
 import org.eclipse.ui.part.IShowInSource;
 import org.eclipse.ui.part.IShowInTarget;
+import org.eclipse.ui.part.PluginTransfer;
 import org.eclipse.ui.part.ResourceTransfer;
 import org.eclipse.ui.part.ShowInContext;
 import org.eclipse.ui.part.ViewPart;
@@ -203,7 +204,7 @@ public class CView extends ViewPart implements ISetSelectionTarget, IPropertyCha
 	 * @see ISetSelectionTarget#selectReveal()
 	 */
 	public void selectReveal(ISelection selection) {
-		IStructuredSelection ssel = SelectionConverter.convertSelectionToCElements(selection);
+		IStructuredSelection ssel = SelectionConverter.convertSelectionToCElements(selection, true);
 		if (!ssel.isEmpty()) {
 			getViewer().setSelection(ssel, true);
 		}
@@ -407,13 +408,14 @@ public class CView extends ViewPart implements ISetSelectionTarget, IPropertyCha
 		Transfer[] transfers= new Transfer[] {
 			LocalSelectionTransfer.getInstance(),
 			ResourceTransfer.getInstance(),
-			FileTransfer.getInstance()};
+			FileTransfer.getInstance(),
+			PluginTransfer.getInstance()};
 		TransferDropTargetListener[] dropListeners= new TransferDropTargetListener[] {
 			new SelectionTransferDropAdapter(viewer),
 			new ResourceTransferDropAdapter(viewer),
 			new FileTransferDropAdapter(viewer)
 		};
-		viewer.addDropSupport(ops, transfers, new DelegatingDropAdapter(dropListeners));
+		viewer.addDropSupport(ops, transfers, new DelegatingDropAdapter(viewer, dropListeners));
 	}
 
 	/**
