@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 IBM Corporation and others.
+ * Copyright (c) 2005, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -8,6 +8,7 @@
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     QNX Software System
+ *     Anton Leherbauer, Wind River Systems, Inc.
  *******************************************************************************/
 package org.eclipse.cdt.internal.ui.editor;
 
@@ -15,6 +16,7 @@ package org.eclipse.cdt.internal.ui.editor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.Iterator;
+import java.util.ResourceBundle;
 
 import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.CCorePreferenceConstants;
@@ -29,20 +31,22 @@ import org.eclipse.cdt.internal.ui.IContextMenuConstants;
 import org.eclipse.cdt.internal.ui.actions.AddBlockCommentAction;
 import org.eclipse.cdt.internal.ui.actions.FoldingActionGroup;
 import org.eclipse.cdt.internal.ui.actions.GoToNextPreviousMemberAction;
+import org.eclipse.cdt.internal.ui.actions.JoinLinesAction;
 import org.eclipse.cdt.internal.ui.actions.RemoveBlockCommentAction;
-import org.eclipse.cdt.internal.ui.browser.typehierarchy.OpenTypeHierarchyAction;
 import org.eclipse.cdt.internal.ui.search.actions.OpenDeclarationsAction;
 import org.eclipse.cdt.internal.ui.search.actions.OpenDefinitionAction;
 import org.eclipse.cdt.internal.ui.search.actions.SelectionSearchGroup;
 import org.eclipse.cdt.internal.ui.text.CPairMatcher;
 import org.eclipse.cdt.internal.ui.text.CSourceViewerConfiguration;
 import org.eclipse.cdt.internal.ui.text.CTextTools;
+import org.eclipse.cdt.internal.ui.text.HTMLTextPresenter;
+import org.eclipse.cdt.internal.ui.text.ICPartitions;
+import org.eclipse.cdt.internal.ui.text.c.hover.SourceViewerInformationControl;
 import org.eclipse.cdt.internal.ui.text.contentassist.ContentAssistPreference;
 import org.eclipse.cdt.internal.ui.util.CUIHelp;
 import org.eclipse.cdt.ui.CUIPlugin;
 import org.eclipse.cdt.ui.IWorkingCopyManager;
 import org.eclipse.cdt.ui.PreferenceConstants;
-import org.eclipse.cdt.ui.actions.RefactoringActionGroup;
 import org.eclipse.cdt.ui.actions.ShowInCViewAction;
 import org.eclipse.cdt.ui.text.folding.ICFoldingStructureProvider;
 import org.eclipse.core.resources.IFile;
@@ -55,25 +59,43 @@ import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IMenuManager;
 import org.eclipse.jface.action.IStatusLineManager;
 import org.eclipse.jface.preference.IPreferenceStore;
+import org.eclipse.jface.text.AbstractInformationControlManager;
 import org.eclipse.jface.text.BadLocationException;
+import org.eclipse.jface.text.DefaultInformationControl;
 import org.eclipse.jface.text.DefaultLineTracker;
 import org.eclipse.jface.text.DocumentCommand;
 import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.IInformationControl;
+import org.eclipse.jface.text.IInformationControlCreator;
 import org.eclipse.jface.text.ILineTracker;
 import org.eclipse.jface.text.IRegion;
+import org.eclipse.jface.text.ITextHover;
 import org.eclipse.jface.text.ITextOperationTarget;
 import org.eclipse.jface.text.ITextSelection;
+import org.eclipse.jface.text.ITextViewer;
+import org.eclipse.jface.text.ITextViewerExtension2;
+import org.eclipse.jface.text.ITextViewerExtension4;
 import org.eclipse.jface.text.ITextViewerExtension5;
 import org.eclipse.jface.text.Position;
 import org.eclipse.jface.text.Region;
+import org.eclipse.jface.text.TextUtilities;
 import org.eclipse.jface.text.contentassist.ContentAssistant;
 import org.eclipse.jface.text.contentassist.IContentAssistant;
+import org.eclipse.jface.text.information.IInformationProvider;
+import org.eclipse.jface.text.information.IInformationProviderExtension;
+import org.eclipse.jface.text.information.IInformationProviderExtension2;
+import org.eclipse.jface.text.information.InformationPresenter;
 import org.eclipse.jface.text.source.Annotation;
+import org.eclipse.jface.text.source.IAnnotationHover;
+import org.eclipse.jface.text.source.IAnnotationHoverExtension;
 import org.eclipse.jface.text.source.IAnnotationModel;
 import org.eclipse.jface.text.source.ICharacterPairMatcher;
+import org.eclipse.jface.text.source.ILineRange;
 import org.eclipse.jface.text.source.ISharedTextColors;
 import org.eclipse.jface.text.source.ISourceViewer;
+import org.eclipse.jface.text.source.ISourceViewerExtension3;
 import org.eclipse.jface.text.source.IVerticalRuler;
+import org.eclipse.jface.text.source.IVerticalRulerInfo;
 import org.eclipse.jface.text.source.SourceViewerConfiguration;
 import org.eclipse.jface.text.source.projection.ProjectionAnnotationModel;
 import org.eclipse.jface.text.source.projection.ProjectionSupport;
@@ -85,10 +107,13 @@ import org.eclipse.jface.viewers.ISelectionProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.search.ui.actions.TextSearchGroup;
+import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IEditorActionBarContributor;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IFileEditorInput;
@@ -113,7 +138,9 @@ import org.eclipse.ui.texteditor.IEditorStatusLine;
 import org.eclipse.ui.texteditor.ITextEditorActionConstants;
 import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
 import org.eclipse.ui.texteditor.MarkerAnnotation;
+import org.eclipse.ui.texteditor.ResourceAction;
 import org.eclipse.ui.texteditor.SourceViewerDecorationSupport;
+import org.eclipse.ui.texteditor.TextEditorAction;
 import org.eclipse.ui.texteditor.TextOperationAction;
 import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
 
@@ -123,6 +150,13 @@ import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
  */
 public class CEditor extends TextEditor implements ISelectionChangedListener, IShowInSource , IReconcilingParticipant{
 
+		
+	/**
+	 * The information provider used to present focusable information
+	 * shells.
+	 */
+	private InformationPresenter fInformationPresenter;
+	
 	/**
 	 * Updates the Java outline page selection and this editor's range indicator.
 	 * 
@@ -139,6 +173,312 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 		}
 	}
 	
+ 	/**
+	 * Information provider used to present focusable information shells.
+	 * 
+	 * @since 3.1.1
+	 */
+	private static final class InformationProvider implements
+			IInformationProvider, IInformationProviderExtension,
+			IInformationProviderExtension2 {
+
+		private IRegion fHoverRegion;
+		private Object fHoverInfo;
+		private IInformationControlCreator fControlCreator;
+
+		InformationProvider(IRegion hoverRegion, Object hoverInfo,
+				IInformationControlCreator controlCreator) {
+			fHoverRegion = hoverRegion;
+			fHoverInfo = hoverInfo;
+			fControlCreator = controlCreator;
+		}
+
+		/*
+		 * @see org.eclipse.jface.text.information.IInformationProvider#getSubject(org.eclipse.jface.text.ITextViewer,
+		 *      int)
+		 */
+		public IRegion getSubject(ITextViewer textViewer, int invocationOffset) {
+			return fHoverRegion;
+		}
+
+		/*
+		 * @see org.eclipse.jface.text.information.IInformationProvider#getInformation(org.eclipse.jface.text.ITextViewer,
+		 *      org.eclipse.jface.text.IRegion)
+		 */
+		public String getInformation(ITextViewer textViewer, IRegion subject) {
+			return fHoverInfo.toString();
+		}
+
+		/*
+		 * @see org.eclipse.jface.text.information.IInformationProviderExtension#getInformation2(org.eclipse.jface.text.ITextViewer,
+		 *      org.eclipse.jface.text.IRegion)
+		 * @since 3.2
+		 */
+		public Object getInformation2(ITextViewer textViewer, IRegion subject) {
+			return fHoverInfo;
+		}
+
+		/*
+		 * @see org.eclipse.jface.text.information.IInformationProviderExtension2#getInformationPresenterControlCreator()
+		 */
+		public IInformationControlCreator getInformationPresenterControlCreator() {
+			return fControlCreator;
+		}
+	}
+
+	/**
+	 * This action behaves in two different ways: If there is no current text
+	 * hover, the tooltip is displayed using information presenter. If there is
+	 * a current text hover, it is converted into a information presenter in
+	 * order to make it sticky.
+	 * 
+	 * @since 3.1.1
+	 */
+	class InformationDispatchAction extends TextEditorAction {
+
+		/** The wrapped text operation action. */
+		private final TextOperationAction fTextOperationAction;
+
+		/**
+		 * Creates a dispatch action.
+		 * 
+		 * @param resourceBundle
+		 *            the resource bundle
+		 * @param prefix
+		 *            the prefix
+		 * @param textOperationAction
+		 *            the text operation action
+		 */
+		public InformationDispatchAction(ResourceBundle resourceBundle,
+				String prefix, final TextOperationAction textOperationAction) {
+			super(resourceBundle, prefix, CEditor.this);
+			if (textOperationAction == null)
+				throw new IllegalArgumentException();
+			fTextOperationAction = textOperationAction;
+		}
+
+		/*
+		 * @see org.eclipse.jface.action.IAction#run()
+		 */
+		public void run() {
+
+			ISourceViewer sourceViewer = getSourceViewer();
+			if (sourceViewer == null) {
+				fTextOperationAction.run();
+				return;
+			}
+
+			if (sourceViewer instanceof ITextViewerExtension4) {
+				ITextViewerExtension4 extension4 = (ITextViewerExtension4) sourceViewer;
+				if (extension4.moveFocusToWidgetToken())
+					return;
+			}
+
+			if (sourceViewer instanceof ITextViewerExtension2) {
+				// does a text hover exist?
+				ITextHover textHover = ((ITextViewerExtension2) sourceViewer)
+						.getCurrentTextHover();
+				if (textHover != null
+						&& makeTextHoverFocusable(sourceViewer, textHover))
+					return;
+			}
+
+			if (sourceViewer instanceof ISourceViewerExtension3) {
+				// does an annotation hover exist?
+				IAnnotationHover annotationHover = ((ISourceViewerExtension3) sourceViewer)
+						.getCurrentAnnotationHover();
+				if (annotationHover != null
+						&& makeAnnotationHoverFocusable(sourceViewer,
+								annotationHover))
+					return;
+			}
+
+			// otherwise, just display the tooltip
+			// fTextOperationAction.run();
+		}
+
+		/**
+		 * Tries to make a text hover focusable (or "sticky").
+		 * 
+		 * @param sourceViewer
+		 *            the source viewer to display the hover over
+		 * @param textHover
+		 *            the hover to make focusable
+		 * @return <code>true</code> if successful, <code>false</code>
+		 *         otherwise
+		 */
+		private boolean makeTextHoverFocusable(ISourceViewer sourceViewer,
+				ITextHover textHover) {
+			Point hoverEventLocation = ((ITextViewerExtension2) sourceViewer)
+					.getHoverEventLocation();
+			int offset = computeOffsetAtLocation(sourceViewer,
+					hoverEventLocation.x, hoverEventLocation.y);
+			if (offset == -1)
+				return false;
+
+			try {
+				IRegion hoverRegion = textHover.getHoverRegion(sourceViewer,
+						offset);
+				if (hoverRegion == null)
+					return false;
+
+				String hoverInfo = textHover.getHoverInfo(sourceViewer,
+						hoverRegion);
+
+				IInformationControlCreator controlCreator = null;
+				if (textHover instanceof IInformationProviderExtension2)
+					controlCreator = ((IInformationProviderExtension2) textHover)
+							.getInformationPresenterControlCreator();
+
+				IInformationProvider informationProvider = new InformationProvider(
+						hoverRegion, hoverInfo, controlCreator);
+
+				fInformationPresenter.setOffset(offset);
+				fInformationPresenter
+						.setAnchor(AbstractInformationControlManager.ANCHOR_BOTTOM);
+				fInformationPresenter.setMargins(6, 6); // default values from
+														// AbstractInformationControlManager
+				String contentType = TextUtilities.getContentType(sourceViewer
+						.getDocument(), ICPartitions.C_PARTITIONING, offset,
+						true);
+				fInformationPresenter.setInformationProvider(
+						informationProvider, contentType);
+				fInformationPresenter.showInformation();
+
+				return true;
+
+			} catch (BadLocationException e) {
+				return false;
+			}
+		}
+
+		/**
+		 * Tries to make an annotation hover focusable (or "sticky").
+		 * 
+		 * @param sourceViewer
+		 *            the source viewer to display the hover over
+		 * @param annotationHover
+		 *            the hover to make focusable
+		 * @return <code>true</code> if successful, <code>false</code>
+		 *         otherwise
+		 */
+		private boolean makeAnnotationHoverFocusable(
+				ISourceViewer sourceViewer, IAnnotationHover annotationHover) {
+			IVerticalRulerInfo info = getVerticalRuler();
+			int line = info.getLineOfLastMouseButtonActivity();
+			if (line == -1)
+				return false;
+
+			try {
+
+				// compute the hover information
+				Object hoverInfo;
+				if (annotationHover instanceof IAnnotationHoverExtension) {
+					IAnnotationHoverExtension extension = (IAnnotationHoverExtension) annotationHover;
+					ILineRange hoverLineRange = extension.getHoverLineRange(
+							sourceViewer, line);
+					if (hoverLineRange == null)
+						return false;
+					final int maxVisibleLines = Integer.MAX_VALUE; // allow any
+																	// number of
+																	// lines
+																	// being
+																	// displayed,
+																	// as we
+																	// support
+																	// scrolling
+					hoverInfo = extension.getHoverInfo(sourceViewer,
+							hoverLineRange, maxVisibleLines);
+				} else {
+					hoverInfo = annotationHover
+							.getHoverInfo(sourceViewer, line);
+				}
+
+				// hover region: the beginning of the concerned line to place
+				// the control right over the line
+				IDocument document = sourceViewer.getDocument();
+				int offset = document.getLineOffset(line);
+				String contentType = TextUtilities.getContentType(document,
+						ICPartitions.C_PARTITIONING, offset, true);
+
+				IInformationControlCreator controlCreator = null;
+
+				/*
+				 * XXX: This is a hack to avoid API changes at the end of 3.2,
+				 * and should be fixed for 3.3, see:
+				 * https://bugs.eclipse.org/bugs/show_bug.cgi?id=137967
+				 */
+				if ("org.eclipse.jface.text.source.projection.ProjectionAnnotationHover".equals(annotationHover.getClass().getName())) { //$NON-NLS-1$
+					controlCreator = new IInformationControlCreator() {
+						public IInformationControl createInformationControl(
+								Shell shell) {
+							int shellStyle = SWT.RESIZE | SWT.TOOL
+									| getOrientation();
+							int style = SWT.V_SCROLL | SWT.H_SCROLL;
+							return new SourceViewerInformationControl(shell,
+									shellStyle, style);
+						}
+					};
+
+				} else {
+					if (annotationHover instanceof IInformationProviderExtension2)
+						controlCreator = ((IInformationProviderExtension2) annotationHover)
+								.getInformationPresenterControlCreator();
+					else if (annotationHover instanceof IAnnotationHoverExtension)
+						controlCreator = ((IAnnotationHoverExtension) annotationHover)
+								.getHoverControlCreator();
+				}
+
+				IInformationProvider informationProvider = new InformationProvider(
+						new Region(offset, 0), hoverInfo, controlCreator);
+
+				fInformationPresenter.setOffset(offset);
+				fInformationPresenter
+						.setAnchor(AbstractInformationControlManager.ANCHOR_RIGHT);
+				fInformationPresenter.setMargins(4, 0); // AnnotationBarHoverManager
+														// sets (5,0), minus
+														// SourceViewer.GAP_SIZE_1
+				fInformationPresenter.setInformationProvider(
+						informationProvider, contentType);
+				fInformationPresenter.showInformation();
+
+				return true;
+
+			} catch (BadLocationException e) {
+				return false;
+			}
+		}
+
+		// modified version from TextViewer
+		private int computeOffsetAtLocation(ITextViewer textViewer, int x, int y) {
+
+			StyledText styledText = textViewer.getTextWidget();
+			IDocument document = textViewer.getDocument();
+
+			if (document == null)
+				return -1;
+
+			try {
+				int widgetOffset = styledText.getOffsetAtLocation(new Point(x,
+						y));
+				Point p = styledText.getLocationAtOffset(widgetOffset);
+				if (p.x > x)
+					widgetOffset--;
+
+				if (textViewer instanceof ITextViewerExtension5) {
+					ITextViewerExtension5 extension = (ITextViewerExtension5) textViewer;
+					return extension.widgetOffset2ModelOffset(widgetOffset);
+				} else {
+					IRegion visibleRegion = textViewer.getVisibleRegion();
+					return widgetOffset + visibleRegion.getOffset();
+				}
+			} catch (IllegalArgumentException e) {
+				return -1;
+			}
+
+		}
+	}
+	
 	/**
 	 * The editor selection changed listener.
 	 * 
@@ -152,8 +492,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 	
 	/** Search actions **/
 	private ActionGroup fSelectionSearchGroup;
-    /** Groups refactoring actions. */
-	private ActionGroup fRefactoringActionGroup;
+	private ActionGroup fTextSearchGroup;
     /** Action which shows selected element in CView. */
 	private ShowInCViewAction fShowInCViewAction;
 	
@@ -572,7 +911,6 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 			fCEditorErrorTickUpdater = null;
 		}
 		
-        final CSourceViewer sourceViewer = (CSourceViewer) getSourceViewer();
         if (fSelectionUpdateListener != null) {
 			getSelectionProvider().addSelectionChangedListener(fSelectionUpdateListener);
 			fSelectionUpdateListener = null;
@@ -599,16 +937,16 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 			fShowInCViewAction = null;
 		}
 		
-		if (fRefactoringActionGroup != null) {
-			fRefactoringActionGroup.dispose();
-			fRefactoringActionGroup = null;
-		}
-		
 		if (fSelectionSearchGroup != null) {
 			fSelectionSearchGroup.dispose();
 			fSelectionSearchGroup = null;
 		}
 
+		if (fTextSearchGroup != null) {
+			fTextSearchGroup.dispose();
+			fTextSearchGroup = null;
+		}
+
 		if (fEditorSelectionChangedListener != null)  {
 			fEditorSelectionChangedListener.uninstall(getSelectionProvider());
 			fEditorSelectionChangedListener= null;
@@ -661,11 +999,23 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 
 		fFoldingGroup= new FoldingActionGroup(this, getSourceViewer());
 
+		// Sticky hover support
+		ResourceAction resAction= new TextOperationAction(CEditorMessages.getResourceBundle(), "ShowToolTip.", this, ISourceViewer.INFORMATION, true); //$NON-NLS-1$
+		ResourceAction resAction2= new InformationDispatchAction(CEditorMessages.getResourceBundle(), "ShowToolTip.", (TextOperationAction) resAction); //$NON-NLS-1$
+		resAction2.setActionDefinitionId(ICEditorActionDefinitionIds.SHOW_TOOLTIP);
+		setAction("ShowToolTip", resAction2); //$NON-NLS-1$
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(resAction2, ICHelpContextIds.SHOW_TOOLTIP_ACTION);
+ 
+		
 		// Default text editing menu items
 		Action action= new GotoMatchingBracketAction(this);
 		action.setActionDefinitionId(ICEditorActionDefinitionIds.GOTO_MATCHING_BRACKET);				
 		setAction(GotoMatchingBracketAction.GOTO_MATCHING_BRACKET, action);
-
+		
+		action = new JoinLinesAction(CEditorMessages.getResourceBundle(), "JoinLines.", this); //$NON-NLS-1$
+		action.setActionDefinitionId(ICEditorActionDefinitionIds.JOIN_LINES);
+		setAction("Join Lines", action); //$NON-NLS-1$
+		
 		action = new TextOperationAction(CEditorMessages.getResourceBundle(), "Comment.", this, ITextOperationTarget.PREFIX); //$NON-NLS-1$
 		action.setActionDefinitionId(ICEditorActionDefinitionIds.COMMENT);
 		setAction("Comment", action); //$NON-NLS-1$
@@ -725,7 +1075,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 		action.setActionDefinitionId(ICEditorActionDefinitionIds.OPEN_CVIEW);
 		setAction("ShowInCView", action); //$NON-NLS-1$
         
-        action = new TextOperationAction(CEditorMessages.getResourceBundle(), "OpenOutline.", this, CSourceViewer.SHOW_OUTLINE); //$NON-NLS-1$
+        action = new TextOperationAction(CEditorMessages.getResourceBundle(), "OpenOutline.", this, CSourceViewer.SHOW_OUTLINE, true); //$NON-NLS-1$
         action.setActionDefinitionId(ICEditorActionDefinitionIds.OPEN_OUTLINE);
         setAction("OpenOutline", action); //$NON-NLS-1$*/
         
@@ -739,7 +1089,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 
         //Assorted action groupings
 		fSelectionSearchGroup = new SelectionSearchGroup(this);
-		fRefactoringActionGroup = new RefactoringActionGroup(this, null);		
+		fTextSearchGroup= new TextSearchGroup(this);
 	}
 
 	/**
@@ -771,9 +1121,8 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 		
 		addAction(menu, IContextMenuConstants.GROUP_GENERATE, "ShowInCView"); //$NON-NLS-1$
 
-        fRefactoringActionGroup.fillContextMenu(menu);
 		fSelectionSearchGroup.fillContextMenu(menu);
-	
+		fTextSearchGroup.fillContextMenu(menu);
 	}
 
 	/**
@@ -810,6 +1159,25 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 	 */
 	public void createPartControl(Composite parent) {
 		super.createPartControl(parent);
+		
+		// Sticky hover support
+		IInformationControlCreator informationControlCreator = new IInformationControlCreator() {
+			public IInformationControl createInformationControl(Shell shell) {
+				boolean cutDown = false;
+				int style = cutDown ? SWT.NONE : (SWT.V_SCROLL | SWT.H_SCROLL);
+				return new DefaultInformationControl(shell, SWT.RESIZE
+						| SWT.TOOL, style, new HTMLTextPresenter(cutDown));
+			}
+		};
+
+		fInformationPresenter = new InformationPresenter(
+				informationControlCreator);
+		fInformationPresenter.setSizeConstraints(60, 10, true, true);
+		fInformationPresenter.install(getSourceViewer());
+		fInformationPresenter
+				.setDocumentPartitioning(ICPartitions.C_PARTITIONING);
+				
+		
 
 		ProjectionViewer projectionViewer= (ProjectionViewer) getSourceViewer();
 		
@@ -854,14 +1222,15 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 	 * 
 	 * @param forward <code>true</code> if search direction is forward, <code>false</code> if backward
 	 */
-	public void gotoAnnotation(boolean forward) {
+	public Annotation gotoAnnotation(boolean forward) {
+		Annotation annotation = null;
 		ITextSelection selection= (ITextSelection) getSelectionProvider().getSelection();
 		Position position= new Position(0, 0);
 		if (false /* delayed - see bug 18316 */) {
 			getNextAnnotation(selection.getOffset(), selection.getLength(), forward, position);
 			selectAndReveal(position.getOffset(), position.getLength());
 		} else /* no delay - see bug 18316 */ {
-			Annotation annotation= getNextAnnotation(selection.getOffset(), selection.getLength(), forward, position);
+			annotation= getNextAnnotation(selection.getOffset(), selection.getLength(), forward, position);
 			setStatusLineErrorMessage(null);
 			setStatusLineMessage(null);
 			if (annotation != null) {
@@ -870,6 +1239,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 				setStatusLineMessage(annotation.getText());
 			}
 		}
+		return annotation;
 	}
 
 	/**
@@ -946,7 +1316,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 	 *         otherwise
 	 * @since 3.0
 	 */
-	private boolean isNavigationTarget(Annotation annotation) {
+	protected boolean isNavigationTarget(Annotation annotation) {
 		Preferences preferences= EditorsUI.getPluginPreferences();
 		AnnotationPreference preference= getAnnotationPreferenceLookup().getAnnotationPreference(annotation);
 //		See bug 41689
@@ -1077,7 +1447,7 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 			try {
 				boolean isProblem= marker.isSubtypeOf(IMarker.PROBLEM);
 				IWorkbenchPage page= getSite().getPage();
-				IViewPart view= page.findView(isProblem ? IPageLayout.ID_PROBLEM_VIEW: IPageLayout.ID_TASK_LIST); //$NON-NLS-1$  //$NON-NLS-2$
+				IViewPart view= page.findView(isProblem ? IPageLayout.ID_PROBLEM_VIEW: IPageLayout.ID_TASK_LIST); 
 				if (view != null) {
 					Method method= view.getClass().getMethod("setSelection", new Class[] { IStructuredSelection.class, boolean.class}); //$NON-NLS-1$
 					method.invoke(view, new Object[] {new StructuredSelection(marker), Boolean.TRUE });
@@ -1443,4 +1813,29 @@ public class CEditor extends TextEditor implements ISelectionChangedListener, IS
 		// CoreModel WorkingCopy changes instead.
 		// It will allow more fined grained.
 	}
+	
+	public CSourceViewer getCSourceViewer()  {
+		ISourceViewer viewer = getSourceViewer();
+		CSourceViewer cViewer = null ;
+		if (viewer instanceof CSourceViewer) {
+			cViewer = (CSourceViewer) viewer;
+		}
+		return cViewer ;
+	}
+	
+	/*
+	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#collectContextMenuPreferencePages()
+	 */
+	protected String[] collectContextMenuPreferencePages() {
+		// Add C/C++ Editor relevant pages
+		String[] parentPrefPageIds = super.collectContextMenuPreferencePages();
+		String[] prefPageIds = new String[parentPrefPageIds.length + 4];
+		int nIds = 0;
+		prefPageIds[nIds++] = "org.eclipse.cdt.ui.preferences.CEditorPreferencePage"; //$NON-NLS-1$
+		prefPageIds[nIds++] = "org.eclipse.cdt.ui.preferences.CodeAssistPreferencePage"; //$NON-NLS-1$
+		prefPageIds[nIds++] = "org.eclipse.cdt.ui.preferences.TemplatePreferencePage"; //$NON-NLS-1$
+		prefPageIds[nIds++] = "org.eclipse.cdt.ui.preferences.CodeFormatterPreferencePage"; //$NON-NLS-1$
+		System.arraycopy(parentPrefPageIds, 0, prefPageIds, nIds, parentPrefPageIds.length);
+		return prefPageIds;
+	}
 }
