@@ -1,5 +1,5 @@
-/**********************************************************************
- * Copyright (c) 2002, 2005 IBM Corporation and others.
+/*******************************************************************************
+ * Copyright (c) 2002, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -17,10 +17,13 @@ import org.eclipse.jface.action.IMenuManager;
 
 public class ExternalSearchEditor extends CEditor {
 	
+	public static final String EDITOR_ID = "org.eclipse.cdt.ui.editor.ExternalSearchEditor";
+	
 	public ExternalSearchEditor(){
 		super();
 		setDocumentProvider(CUIPlugin.getDefault().getExternalSearchDocumentProvider());
 	}
+	
 	public void editorContextMenuAboutToShow(IMenuManager menu) {
 	  super.editorContextMenuAboutToShow(menu);
 	  IContributionItem[] contrItem = menu.getItems();
