@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 QNX Software Systems and others.
+ * Copyright (c) 2004, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,7 @@
  *
  * Contributors:
  *     QNX Software Systems - initial API and implementation
+ *     IBM Corporation
  *******************************************************************************/
 package org.eclipse.cdt.internal.ui.wizards.classwizard;
 
@@ -163,7 +164,11 @@ public class NewClassCodeGenerator {
 	            headerWorkingCopy.commit(true, monitor);
 	            monitor.worked(50);
 	
-	            createdClass = headerWorkingCopy.getElement(fClassName.toString());
+	            IQualifiedTypeName className = new QualifiedTypeName(fClassName);
+	            if(fNamespace != null){
+	            	className = fNamespace.append(className);
+	            }
+	            createdClass = headerWorkingCopy.getElement(className.getFullyQualifiedName());
 	            fCreatedClass = createdClass;
 	            fCreatedHeaderTU = headerTU;
             }
@@ -522,10 +527,10 @@ public class NewClassCodeGenerator {
                 if (cProject.getPath().segment(0).equals(folderToAdd.segment(0)))
                 	continue;
                 
-                IProject includeProject = PathUtil.getEnclosingProject(folderToAdd);
+                ICProject includeProject = PathUtil.getEnclosingProject(folderToAdd);
 
                 // make sure that the include is made the same way that build properties for projects makes them, so .contains below is a valid check
-                IIncludeEntry entry = CoreModel.newIncludeEntry(addToResourcePath, null, includeProject.getLocation(), true);
+                IIncludeEntry entry = CoreModel.newIncludeEntry(addToResourcePath, null, includeProject.getProject().getLocation(), true);
                 
                 if (!checkEntryList.contains(entry)) // if the path already exists in the #includes then don't add it
                 	pathEntryList.add(entry);
