@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004 QNX Software Systems and others.
+ * Copyright (c) 2004, 2006 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,7 +10,6 @@
  *******************************************************************************/
 package org.eclipse.cdt.internal.ui.wizards.filewizard;
 
-import org.eclipse.cdt.core.browser.PathUtil;
 import org.eclipse.cdt.core.model.CModelException;
 import org.eclipse.cdt.core.model.CoreModel;
 import org.eclipse.cdt.core.model.ICContainer;
@@ -31,6 +30,7 @@ import org.eclipse.cdt.internal.ui.wizards.dialogfields.LayoutUtil;
 import org.eclipse.cdt.internal.ui.wizards.dialogfields.Separator;
 import org.eclipse.cdt.internal.ui.wizards.dialogfields.StringButtonDialogField;
 import org.eclipse.cdt.ui.CUIPlugin;
+import org.eclipse.cdt.utils.PathUtil;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
@@ -337,7 +337,6 @@ public abstract class AbstractFileCreationWizardPage extends NewElementWizardPag
 		    IPath oldFolderPath = getSourceFolderFullPath();
 			IPath newFolderPath = chooseSourceFolder(oldFolderPath);
 			if (newFolderPath != null) {
-				IPath filePath = getFileFullPath();
 				setSourceFolderFullPath(newFolderPath, false);
 				handleFieldChanged(ALL_FIELDS);
 			}
@@ -527,7 +526,7 @@ public abstract class AbstractFileCreationWizardPage extends NewElementWizardPag
 	private ICElement getSourceFolderFromPath(IPath path) {
 	    if (path == null)
 	        return null;
-	    while (!path.isEmpty()) {
+	    while (path.segmentCount() > 0) {
 		    IResource res = fWorkspaceRoot.findMember(path);
 			if (res != null && res.exists()) {
 				int resType = res.getType();
